// This code was generated by a tool.
// Processing tool available at https://github.com/mezoni/peg

library preprocessor.internal.parser.p_parser;

import "dart:math";

import "package:macro_processor/internal/ast/ast.dart";

Expression _binary(left, _Node node, int position) {
  if (node == null) {
    return left;
  }
  var right = _binary(node.operand, node.next, position);
  return new BinaryExpression(left: left, operator: node.operator, position: position, right: right);  
 }

Expression _conditional(Expression condition, Expression success, Expression fail, int position) {  
  return new ConditionalExpression(condition: condition, success: success, fail: fail, position: position);
}

int _escape(String ch) {
  switch(ch) {
    case "a":
      return 0x07;
    case "b":
      return 0x08;
    case "f":
      return 0x0c;
    case "n":
      return 0x0a;
    case "r":
      return 0x0d;
    case "t":
      return 0x09;
    case "v":
      return 0x0b;
    case "\'":
      return 0x27;
    case "\"":
      return 0x22;
    case "\?":
      return 0x3f;
    default:
      throw new StateError("Unknown escape sequence: \\$ch");  
  }
}

FloatingPointLiteral _floatLiteral(String source, int position) {
  var value = _parserFloat(source);
  return new FloatingPointLiteral(position: position, text: source, value: value);
}

double _parserFloat(String source) {
  source = source.toLowerCase();
  var hex = false;
  if (source.startsWith("0x")) {
    source = source.substring(2);
    hex = true;
  }
  var length = source.length;
  var endsWith = source[length - 1];
  if (endsWith == "f" || endsWith == "l") {
    source = source.substring(0, length - 1);
  }
  if (!hex) {
    return double.parse(source);
  }
  var index = source.indexOf("p");
  var exponent = int.parse(source.substring(index + 1));
  source = source.substring(0, index);
  index = source.indexOf(".");
  int integer = 0;
  int fractional = 0;
  if (index == 0) {
    fractional = int.parse(source.substring(1), radix: 16);
  } else if (index > 0) {
    integer = int.parse(source.substring(0, index), radix: 16);
    fractional = int.parse(source.substring(index + 1), radix: 16);
  } else {
    integer = int.parse(source, radix: 16);
  }
  var d = double.parse(".$fractional") * .625;
  return (integer + d) * pow(2, exponent);
}

StringLiteral _stringLiteral(String source, List<int> characters, int position) {
  var value = "";
  if (characters != null) {
    value = new String.fromCharCodes(characters);
  }    
  return new StringLiteral(position: position, text: source, value: value);
}

SourceLine _sourceLine(List<SourceFragment> fragments, SourceFragment last, int position) {
  if (last != null) {
    fragments.add(last);
  }
  return new SourceLine(fragments: fragments, position: position); 
}

Expression _unary(String operator, Expression operand, int position) {  
  return new UnaryExpression(operand: operand, operator: operator, position: position);
}

class _Node {
  final _Node next;

  final dynamic operand;
  
  final String operator;
  
  final int position;  
  
  _Node(this.operator, this.operand, this.next, this.position);
  
  String toString() {
    var sb = new StringBuffer();
    sb.write(operator);
    sb.write(" ");
    sb.write(operand);
    if (next != null) {
      sb.write(" ");
      sb.write(next);    
    }
    return sb.toString();
  }
}

class PParser {
  static final List<String> _ascii = new List<String>.generate(128, (c) => new String.fromCharCode(c));
  
  static final List<String> _expect0 = <String>["eof"];
  
  static final List<String> _expect1 = <String>["\'#if\'", "\'#ifdef\'", "\'#ifndef\'", "control_line", "text_line"];
  
  static final List<String> _expect10 = <String>[null, "\'&\'"];
  
  static final List<String> _expect100 = <String>["sign"];
  
  static final List<String> _expect101 = <String>["simple_escape_sequence"];
  
  static final List<String> _expect102 = <String>["\'/\'"];
  
  static final List<String> _expect103 = <String>["source_fragment"];
  
  static final List<String> _expect104 = <String>[null];
  
  static final List<String> _expect105 = <String>["source_line"];
  
  static final List<String> _expect106 = <String>["spaces"];
  
  static final List<String> _expect107 = <String>["string_literal_base"];
  
  static final List<String> _expect108 = <String>["\'~\'"];
  
  static final List<String> _expect109 = <String>["\'#undef\'"];
  
  static final List<String> _expect11 = <String>[null, "\'^\'"];
  
  static final List<String> _expect110 = <String>["universal_character_name"];
  
  static final List<String> _expect111 = <String>["unsigned_suffix"];
  
  static final List<String> _expect12 = <String>[null, "\'|\'"];
  
  static final List<String> _expect13 = <String>[null, "\'&&\'"];
  
  static final List<String> _expect14 = <String>[null, "\'||\'"];
  
  static final List<String> _expect15 = <String>["\'#elif\'"];
  
  static final List<String> _expect16 = <String>["\'#else\'"];
  
  static final List<String> _expect17 = <String>["eof", "new_line"];
  
  static final List<String> _expect18 = <String>["\'+\'"];
  
  static final List<String> _expect19 = <String>["\'&\'"];
  
  static final List<String> _expect2 = <String>["\'#if\'", "\'#ifdef\'", "\'#ifndef\'"];
  
  static final List<String> _expect20 = <String>["\'&&\'"];
  
  static final List<String> _expect21 = <String>["constant"];
  
  static final List<String> _expect22 = <String>["control_line"];
  
  static final List<String> _expect23 = <String>["\'defined\'"];
  
  static final List<String> _expect24 = <String>["\'#endif\'"];
  
  static final List<String> _expect25 = <String>["\'==\'"];
  
  static final List<String> _expect26 = <String>["identifier"];
  
  static final List<String> _expect27 = <String>["\'#if\'"];
  
  static final List<String> _expect28 = <String>["\'#ifdef\'"];
  
  static final List<String> _expect29 = <String>["\'#ifndef\'"];
  
  static final List<String> _expect3 = <String>["\'(\'", "\'defined\'", "constant", "identifier", "unary_operator"];
  
  static final List<String> _expect30 = <String>["\'(\'"];
  
  static final List<String> _expect31 = <String>["multiplicative_operator"];
  
  static final List<String> _expect32 = <String>["\'!=\'"];
  
  static final List<String> _expect33 = <String>["new_line"];
  
  static final List<String> _expect34 = <String>["\'?\'"];
  
  static final List<String> _expect35 = <String>["relational_operator"];
  
  static final List<String> _expect36 = <String>["\')\'"];
  
  static final List<String> _expect37 = <String>["\':\'"];
  
  static final List<String> _expect38 = <String>["\'<<\'"];
  
  static final List<String> _expect39 = <String>["text_line"];
  
  static final List<String> _expect4 = <String>["\'(\'", "\'defined\'", "constant", "identifier"];
  
  static final List<String> _expect40 = <String>["unary_operator"];
  
  static final List<String> _expect41 = <String>["\'|\'"];
  
  static final List<String> _expect42 = <String>["\'||\'"];
  
  static final List<String> _expect43 = <String>["\'^\'"];
  
  static final List<String> _expect44 = <String>["any_char_until_eol"];
  
  static final List<String> _expect45 = <String>["any_chars_until_eol"];
  
  static final List<String> _expect46 = <String>["\'*\'"];
  
  static final List<String> _expect47 = <String>["binary_exponent_part"];
  
  static final List<String> _expect48 = <String>["P", "p"];
  
  static final List<String> _expect49 = <String>[null, "sign"];
  
  static final List<String> _expect5 = <String>[null, "multiplicative_operator"];
  
  static final List<String> _expect50 = <String>["c_char"];
  
  static final List<String> _expect51 = <String>["character_constant_base"];
  
  static final List<String> _expect52 = <String>["constant_base"];
  
  static final List<String> _expect53 = <String>["decimal_constant"];
  
  static final List<String> _expect54 = <String>["decimal_constant1"];
  
  static final List<String> _expect55 = <String>["decimal_floating_constant"];
  
  static final List<String> _expect56 = <String>["decimal_floating_constant_base"];
  
  static final List<String> _expect57 = <String>["\'#define\'"];
  
  static final List<String> _expect58 = <String>["digit"];
  
  static final List<String> _expect59 = <String>["digit_sequence"];
  
  static final List<String> _expect6 = <String>[null, "\'+\'"];
  
  static final List<String> _expect60 = <String>["\'#error\'"];
  
  static final List<String> _expect61 = <String>["escape_sequence"];
  
  static final List<String> _expect62 = <String>["\'!\'"];
  
  static final List<String> _expect63 = <String>["exponent_part"];
  
  static final List<String> _expect64 = <String>["E", "e"];
  
  static final List<String> _expect65 = <String>["floating_constant_base"];
  
  static final List<String> _expect66 = <String>["floating_suffix"];
  
  static final List<String> _expect67 = <String>["fractional_constant"];
  
  static final List<String> _expect68 = <String>["\'>\'"];
  
  static final List<String> _expect69 = <String>["\'>=\'"];
  
  static final List<String> _expect7 = <String>[null, "\'<<\'"];
  
  static final List<String> _expect70 = <String>["hex_quad"];
  
  static final List<String> _expect71 = <String>["hexadecimal_constant"];
  
  static final List<String> _expect72 = <String>["hexadecimal_constant1"];
  
  static final List<String> _expect73 = <String>["hexadecimal_digit"];
  
  static final List<String> _expect74 = <String>["hexadecimal_digit_sequence"];
  
  static final List<String> _expect75 = <String>["\'\\x\'"];
  
  static final List<String> _expect76 = <String>["hexadecimal_escape_sequence1"];
  
  static final List<String> _expect77 = <String>["hexadecimal_floating_constant"];
  
  static final List<String> _expect78 = <String>["hexadecimal_fractional_constant"];
  
  static final List<String> _expect79 = <String>["hexadecimal_prefix"];
  
  static final List<String> _expect8 = <String>[null, "relational_operator"];
  
  static final List<String> _expect80 = <String>["identifier_base"];
  
  static final List<String> _expect81 = <String>["identifier_base1"];
  
  static final List<String> _expect82 = <String>["identifier_nondigit"];
  
  static final List<String> _expect83 = <String>["integer_constant_base"];
  
  static final List<String> _expect84 = <String>["integer_suffix"];
  
  static final List<String> _expect85 = <String>["long_long_suffix"];
  
  static final List<String> _expect86 = <String>["long_suffix"];
  
  static final List<String> _expect87 = <String>["\'<\'"];
  
  static final List<String> _expect88 = <String>["\'<=\'"];
  
  static final List<String> _expect89 = <String>["new_line_fragment"];
  
  static final List<String> _expect9 = <String>[null, "\'!=\'", "\'==\'"];
  
  static final List<String> _expect90 = <String>["nondigit"];
  
  static final List<String> _expect91 = <String>["nonzero_digit"];
  
  static final List<String> _expect92 = <String>["\'0\'"];
  
  static final List<String> _expect93 = <String>["octal_constant1"];
  
  static final List<String> _expect94 = <String>["octal_digit"];
  
  static final List<String> _expect95 = <String>["\'\\\'"];
  
  static final List<String> _expect96 = <String>["\'%\'"];
  
  static final List<String> _expect97 = <String>["replacement_list"];
  
  static final List<String> _expect98 = <String>["s_char"];
  
  static final List<String> _expect99 = <String>["s_char_sequence"];
  
  static final List<bool> _lookahead = _unmap([0x401, 0x0, 0x0, 0x3ff4983, 0x2ffffffc, 0x1ffffffa, 0x701ffa0e, 0x697fffff, 0x7fffffff, 0x7e87ffff, 0x7fdfffff, 0x7fffffff, 0xc3ff, 0xc000000, 0x10000020, 0x20c0, 0x7f40c100, 0x600007, 0x30000000, 0x40000000, 0x421]);
  
  // '\n', '\r'
  static final List<bool> _mapping0 = _unmap([0x9]);
  
  // '\n', '\r', '\'', '\\'
  static final List<bool> _mapping1 = _unmap([0x20000009, 0x0, 0x100000]);
  
  // 'U', 'u'
  static final List<bool> _mapping10 = _unmap([0x1, 0x2]);
  
  // 'F', 'L', 'f', 'l'
  static final List<bool> _mapping2 = _unmap([0x41, 0x82]);
  
  // '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', ':', ';', '<', '=', '>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^', '_', '`', 'a', 'f'
  static final List<bool> _mapping3 = _unmap([0x7fffffff, 0x87ffff]);
  
  // 'L', 'l'
  static final List<bool> _mapping4 = _unmap([0x1, 0x2]);
  
  // 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '_', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'
  static final List<bool> _mapping5 = _unmap([0x43ffffff, 0x7fffffe]);
  
  // '\n', '\r', '\"', '\\'
  static final List<bool> _mapping6 = _unmap([0x1000009, 0x0, 0x100000]);
  
  // '+', '-'
  static final List<bool> _mapping7 = _unmap([0x5]);
  
  // '\"', '\'', '?', '\\', 'a', 'b', 'f', 'n', 'r', 't', 'v'
  static final List<bool> _mapping8 = _unmap([0x20000021, 0x8000000, 0x544046]);
  
  // '\t', ' '
  static final List<bool> _mapping9 = _unmap([0x800001]);
  
  // '&&'
  static final List<int> _strings0 = <int>[38, 38];
  
  // '#elif'
  static final List<int> _strings1 = <int>[35, 101, 108, 105, 102];
  
  // '||'
  static final List<int> _strings10 = <int>[124, 124];
  
  // 'defined'
  static final List<int> _strings11 = <int>[100, 101, 102, 105, 110, 101, 100];
  
  // 'L''
  static final List<int> _strings12 = <int>[76, 39];
  
  // '#define'
  static final List<int> _strings13 = <int>[35, 100, 101, 102, 105, 110, 101];
  
  // '#error'
  static final List<int> _strings14 = <int>[35, 101, 114, 114, 111, 114];
  
  // '>='
  static final List<int> _strings15 = <int>[62, 61];
  
  // '\x'
  static final List<int> _strings16 = <int>[92, 120];
  
  // '0x'
  static final List<int> _strings17 = <int>[48, 120];
  
  // '0X'
  static final List<int> _strings18 = <int>[48, 88];
  
  // 'LL'
  static final List<int> _strings19 = <int>[76, 76];
  
  // '#else'
  static final List<int> _strings2 = <int>[35, 101, 108, 115, 101];
  
  // 'll'
  static final List<int> _strings20 = <int>[108, 108];
  
  // '<<'
  static final List<int> _strings21 = <int>[60, 60];
  
  // '<='
  static final List<int> _strings22 = <int>[60, 61];
  
  // 'L"'
  static final List<int> _strings23 = <int>[76, 34];
  
  // '#undef'
  static final List<int> _strings24 = <int>[35, 117, 110, 100, 101, 102];
  
  // '\U'
  static final List<int> _strings25 = <int>[92, 85];
  
  // '\u'
  static final List<int> _strings26 = <int>[92, 117];
  
  // '#endif'
  static final List<int> _strings3 = <int>[35, 101, 110, 100, 105, 102];
  
  // '=='
  static final List<int> _strings4 = <int>[61, 61];
  
  // '#if'
  static final List<int> _strings5 = <int>[35, 105, 102];
  
  // '#ifdef'
  static final List<int> _strings6 = <int>[35, 105, 102, 100, 101, 102];
  
  // '#ifndef'
  static final List<int> _strings7 = <int>[35, 105, 102, 110, 100, 101, 102];
  
  // '!='
  static final List<int> _strings8 = <int>[33, 61];
  
  // '\r\n'
  static final List<int> _strings9 = <int>[13, 10];
  
  final List<int> _tokenFlags = [1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0];
  
  final List<String> _tokenNames = ["identifier", "\'(\'", "new_line", "\')\'", "\'defined\'", "any_char_until_eol", "any_chars_until_eol", "\'*\'", "binary_exponent_part", "c_char", "character_constant_base", "constant_base", "decimal_constant", "decimal_constant1", "decimal_floating_constant", "decimal_floating_constant_base", "\'#define\'", "\'#define\'", "digit", "digit_sequence", "\'#error\'", "\'#error\'", "escape_sequence", "\'!\'", "exponent_part", "floating_constant_base", "floating_suffix", "fractional_constant", "\'>\'", "\'>=\'", "hex_quad", "hexadecimal_constant", "hexadecimal_constant1", "hexadecimal_digit", "hexadecimal_digit_sequence", "\'\\x\'", "hexadecimal_escape_sequence1", "hexadecimal_floating_constant", "hexadecimal_fractional_constant", "hexadecimal_prefix", "identifier_base", "identifier_base1", "identifier_nondigit", "integer_constant_base", "integer_suffix", "long_long_suffix", "long_suffix", "\'<<\'", "\'<\'", "\'<=\'", "\'+\'", "new_line_fragment", "nondigit", "nonzero_digit", "\'0\'", "octal_constant1", "octal_digit", "\'\\\'", "\'%\'", "\'+\'", "replacement_list", "\'<<\'", "s_char", "s_char_sequence", "sign", "simple_escape_sequence", "\'/\'", "source_fragment", "source_line", "spaces", "string_literal_base", "\'~\'", "\'#undef\'", "\'#undef\'", "universal_character_name", "unsigned_suffix"];
  
  static final List<List<int>> _transitions0 = [[0, 34, 36, 1114111], [35, 35]];
  
  static final List<List<int>> _transitions1 = [[33, 33, 39, 40, 43, 43, 46, 46, 48, 57, 65, 90, 92, 92, 95, 95, 97, 122, 126, 126]];
  
  static final List<List<int>> _transitions10 = [[0, 93, 95, 1114111], [94, 94]];
  
  static final List<List<int>> _transitions11 = [[0, 123, 125, 1114111], [124, 124]];
  
  static final List<List<int>> _transitions12 = [[0, 9, 11, 12, 14, 1114111], [10, 10, 13, 13]];
  
  static final List<List<int>> _transitions13 = [[39, 39, 46, 46, 48, 57, 76, 76]];
  
  static final List<List<int>> _transitions14 = [[65, 90, 92, 92, 95, 95, 97, 122]];
  
  static final List<List<int>> _transitions15 = [[37, 37], [42, 42], [47, 47]];
  
  static final List<List<int>> _transitions16 = [[10, 10], [13, 13]];
  
  static final List<List<int>> _transitions17 = [[60, 60], [62, 62]];
  
  static final List<List<int>> _transitions18 = [[33, 33], [43, 43], [126, 126]];
  
  static final List<List<int>> _transitions19 = [[80, 80, 112, 112]];
  
  static final List<List<int>> _transitions2 = [[33, 33, 43, 43, 126, 126], [39, 40, 46, 46, 48, 57, 65, 90, 92, 92, 95, 95, 97, 122]];
  
  static final List<List<int>> _transitions20 = [[80, 80], [112, 112]];
  
  static final List<List<int>> _transitions21 = [[0, 42, 44, 44, 46, 1114111], [43, 43, 45, 45]];
  
  static final List<List<int>> _transitions22 = [[39, 39], [76, 76]];
  
  static final List<List<int>> _transitions23 = [[39, 39, 76, 76], [46, 46], [48, 57]];
  
  static final List<List<int>> _transitions24 = [[0, 47, 58, 1114111], [48, 57]];
  
  static final List<List<int>> _transitions25 = [[46, 46, 48, 57]];
  
  static final List<List<int>> _transitions26 = [[46, 46], [48, 57]];
  
  static final List<List<int>> _transitions27 = [[69, 69, 101, 101]];
  
  static final List<List<int>> _transitions28 = [[69, 69], [101, 101]];
  
  static final List<List<int>> _transitions29 = [[70, 70, 76, 76, 102, 102, 108, 108]];
  
  static final List<List<int>> _transitions3 = [[39, 39, 46, 46, 48, 57], [40, 40], [65, 90], [92, 92, 95, 95, 97, 99, 101, 122], [100, 100]];
  
  static final List<List<int>> _transitions30 = [[48, 97, 102, 102]];
  
  static final List<List<int>> _transitions31 = [[0, 47, 98, 101, 103, 1114111], [48, 97, 102, 102]];
  
  static final List<List<int>> _transitions32 = [[46, 46], [48, 97, 102, 102]];
  
  static final List<List<int>> _transitions33 = [[0, 47, 58, 64, 91, 91, 93, 94, 96, 96, 123, 1114111], [48, 57], [65, 90, 92, 92, 95, 95, 97, 122]];
  
  static final List<List<int>> _transitions34 = [[65, 90, 95, 95, 97, 122], [92, 92]];
  
  static final List<List<int>> _transitions35 = [[48, 48], [49, 57]];
  
  static final List<List<int>> _transitions36 = [[76, 76, 108, 108], [85, 85, 117, 117]];
  
  static final List<List<int>> _transitions37 = [[76, 76], [108, 108]];
  
  static final List<List<int>> _transitions38 = [[76, 76, 108, 108]];
  
  static final List<List<int>> _transitions39 = [[10, 10, 13, 13]];
  
  static final List<List<int>> _transitions4 = [[0, 36, 38, 41, 43, 46, 48, 1114111], [37, 37, 42, 42, 47, 47]];
  
  static final List<List<int>> _transitions40 = [[65, 90, 95, 95, 97, 122]];
  
  static final List<List<int>> _transitions41 = [[0, 47, 56, 1114111], [48, 55]];
  
  static final List<List<int>> _transitions42 = [[43, 43, 45, 45]];
  
  static final List<List<int>> _transitions43 = [[0, 33, 35, 38, 40, 45, 47, 47, 58, 64, 91, 91, 93, 94, 96, 96, 123, 1114111], [34, 34], [39, 39, 46, 46, 48, 57], [65, 75, 77, 90, 92, 92, 95, 95, 97, 122], [76, 76]];
  
  static final List<List<int>> _transitions44 = [[34, 34], [76, 76]];
  
  static final List<List<int>> _transitions45 = [[85, 85, 117, 117]];
  
  static final List<List<int>> _transitions5 = [[0, 42, 44, 1114111], [43, 43]];
  
  static final List<List<int>> _transitions6 = [[0, 59, 61, 1114111], [60, 60]];
  
  static final List<List<int>> _transitions7 = [[0, 59, 61, 61, 63, 1114111], [60, 60, 62, 62]];
  
  static final List<List<int>> _transitions8 = [[0, 32, 34, 60, 62, 1114111], [33, 33], [61, 61]];
  
  static final List<List<int>> _transitions9 = [[0, 37, 39, 1114111], [38, 38]];
  
  List<Map<int, List>> _cache;
  
  List<int> _cachePos;
  
  List<bool> _cacheable;
  
  int _ch;
  
  int _cursor;
  
  List<PParserError> _errors;
  
  List<String> _expected;
  
  int _failurePos;
  
  List<int> _input;
  
  int _inputLen;
  
  int _startPos;
  
  int _testing;
  
  int _token;
  
  int _tokenLevel;
  
  int _tokenStart;
  
  bool success;
  
  final String text;
  
  PParser(this.text) {
    if (text == null) {
      throw new ArgumentError('text: $text');
    }    
    _input = _toCodePoints(text);
    _inputLen = _input.length;    
    reset(0);    
  }
  
  void _addToCache(dynamic result, int start, int id) {
    var map = _cache[id];
    if (map == null) {
      map = <int, List>{};
      _cache[id] = map;
    }
    map[start] = [result, _cursor, success];    
  }
  
  void _beginToken(int tokenId) {
    if (_tokenLevel++ == 0) {
      _token = tokenId;
      _tokenStart = _cursor;
    }  
  }
  
  void _endToken() {
    if (--_tokenLevel == 0) {
      _token = null;
      _tokenStart = null;
    }    
  }
  
  void _failure([List<String> expected]) {  
    if (_failurePos > _cursor) {
      return;
    }
    if (_failurePos < _cursor) {    
      _expected = [];
     _failurePos = _cursor;
    }
    if (_token != null) {
      var flag = _tokenFlags[_token];
      var name = _tokenNames[_token];
      if (_failurePos == _inputLen && (flag & 1) != 0) {             
        var message = "Unterminated $name";
        _errors.add(new PParserError(PParserError.UNTERMINATED, _failurePos, _tokenStart, message));
        _expected.addAll(expected);            
      } else if (_failurePos > _tokenStart && (flag & 1) != 0) {             
        var message = "Malformed $name";
        _errors.add(new PParserError(PParserError.MALFORMED, _failurePos, _tokenStart, message));
        _expected.addAll(expected);            
      } else {
        _expected.add(name);
      }            
    } else if (expected == null) {
      _expected.add(null);
    } else {
      _expected.addAll(expected);
    }   
  }
  
  List _flatten(dynamic value) {
    if (value is List) {
      var result = [];
      var length = value.length;
      for (var i = 0; i < length; i++) {
        var element = value[i];
        if (element is Iterable) {
          result.addAll(_flatten(element));
        } else {
          result.add(element);
        }
      }
      return result;
    } else if (value is Iterable) {
      var result = [];
      for (var element in value) {
        if (element is! List) {
          result.add(element);
        } else {
          result.addAll(_flatten(element));
        }
      }
    }
    return [value];
  }
  
  dynamic _getFromCache(int id) {  
    if (!_cacheable[id]) {  
      _cacheable[id] = true;  
      return null;
    }
    var map = _cache[id];
    if (map == null) {
      return null;
    }
    var data = map[_cursor];
    if (data == null) {
      return null;
    }
    _cursor = data[1];
    success = data[2];
    if (_cursor < _inputLen) {
      _ch = _input[_cursor];
    } else {
      _ch = -1;
    }
    return data;  
  }
  
  int _getState(List<List<int>> transitions) {
    var count = transitions.length;
    var state = 0;
    for ( ; state < count; state++) {
      var found = false;
      var ranges = transitions[state];    
      while (true) {
        var right = ranges.length ~/ 2;
        if (right == 0) {
          break;
        }
        var left = 0;
        if (right == 1) {
          if (_ch <= ranges[1] && _ch >= ranges[0]) {
            found = true;          
          }
          break;
        }
        int middle;
        while (left < right) {
          middle = (left + right) >> 1;
          var index = middle << 1;
          if (ranges[index + 1] < _ch) {
            left = middle + 1;
          } else {
            if (_ch >= ranges[index]) {
              found = true;
              break;
            }
            right = middle;
          }
        }
        break;
      }
      if (found) {
        return state; 
      }   
    }
    if (_ch != -1) {
      return state;
    }
    return state + 1;  
  }
  
  List _list(Object first, List next) {
    var length = next.length;
    var list = new List(length + 1);
    list[0] = first;
    for (var i = 0; i < length; i++) {
      list[i + 1] = next[i][1];
    }
    return list;
  }
  
  String _matchAny() {
    success = _cursor < _inputLen;
    if (success) {
      String result;
      if (_ch < 128) {
        result = _ascii[_ch];  
      } else {
        result = new String.fromCharCode(_ch);
      }    
      if (++_cursor < _inputLen) {
        _ch = _input[_cursor];
      } else {
        _ch = -1;
      }    
      return result;
    }    
    return null;  
  }
  
  String _matchChar(int ch, String string) {
    success = _ch == ch;
    if (success) {
      var result = string;  
      if (++_cursor < _inputLen) {
        _ch = _input[_cursor];
      } else {
        _ch = -1;
      }    
      return result;
    }  
    return null;  
  }
  
  String _matchMapping(int start, int end, List<bool> mapping) {
    success = _ch >= start && _ch <= end;
    if (success) {    
      if(mapping[_ch - start]) {
        String result;
        if (_ch < 128) {
          result = _ascii[_ch];  
        } else {
          result = new String.fromCharCode(_ch);
        }     
        if (++_cursor < _inputLen) {
          _ch = _input[_cursor];
        } else {
          _ch = -1;
        }      
        return result;
      }
      success = false;
    }  
    return null;  
  }
  
  String _matchRange(int start, int end) {
    success = _ch >= start && _ch <= end;
    if (success) {
      String result;
      if (_ch < 128) {
        result = _ascii[_ch];  
      } else {
        result = new String.fromCharCode(_ch);
      }        
      if (++_cursor < _inputLen) {
        _ch = _input[_cursor];
      } else {
        _ch = -1;
      }  
      return result;
    }  
    return null;  
  }
  
  String _matchRanges(List<int> ranges) {
    var length = ranges.length;
    for (var i = 0; i < length; i += 2) {    
      if (_ch >= ranges[i]) {
        if (_ch <= ranges[i + 1]) {
          String result;
          if (_ch < 128) {
            result = _ascii[_ch];  
          } else {
            result = new String.fromCharCode(_ch);
          }          
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
             _ch = -1;
          }
          success = true;    
          return result;
        }      
      } else break;  
    }
    success = false;  
    return null;  
  }
  
  String _matchString(List<int> codePoints, String string) {
    var length = codePoints.length;  
    success = _cursor + length <= _inputLen;
    if (success) {
      for (var i = 0; i < length; i++) {
        if (codePoints[i] != _input[_cursor + i]) {
          success = false;
          break;
        }
      }
    } else {
      success = false;
    }  
    if (success) {
      _cursor += length;      
      if (_cursor < _inputLen) {
        _ch = _input[_cursor];
      } else {
        _ch = -1;
      }    
      return string;      
    }  
    return null; 
  }
  
  void _nextChar() {
    if (++_cursor < _inputLen) {
      _ch = _input[_cursor];
    } else {
      _ch = -1;
    }  
  }
  
  dynamic _parse_DEFINED() {
    // MORPHEME
    // DEFINED <- 'defined' spaces
    var $$;
    _beginToken(4);  
    // => 'defined' spaces # Choice
    switch (_ch == 100 ? 0 : _ch == -1 ? 2 : 1) {
      // [d]
      case 0:
        // => 'defined' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => 'defined'
          $$ = _matchString(_strings11, 'defined');
          // <= 'defined'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // 'defined'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= 'defined' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined'
      _failure(_expect23);
    }
    // <= 'defined' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_additive_expression() {
    // NONTERMINAL
    // additive_expression <- multiplicative_expression additive_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[15] >= pos) {
      $$ = _getFromCache(15);
    } else {
      _cachePos[15] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => multiplicative_expression additive_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => multiplicative_expression additive_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => multiplicative_expression
          $$ = _parse_multiplicative_expression();
          // <= multiplicative_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => additive_expression1
          $$ = _parse_additive_expression1();
          // <= additive_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // multiplicative_expression
            final $1 = seq[0];
            // additive_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= multiplicative_expression additive_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= multiplicative_expression additive_expression1 # Choice
    if (_cacheable[15]) {
      _addToCache($$, pos, 15);
    }  
    return $$;
  }
  
  dynamic _parse_additive_expression1() {
    // NONTERMINAL
    // additive_expression1 <- additive_operator multiplicative_expression additive_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[21] >= pos) {
      $$ = _getFromCache(21);
    } else {
      _cachePos[21] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => additive_operator multiplicative_expression additive_expression1 / '' # Choice
    switch (_getState(_transitions5)) {
      // [\u0000-*] [,-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [+]
      case 1:
        while (true) {
          // => additive_operator multiplicative_expression additive_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => additive_operator
            $$ = _parse_additive_operator();
            // <= additive_operator
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => multiplicative_expression
            $$ = _parse_multiplicative_expression();
            // <= multiplicative_expression
            if (!success) break;
            seq[1] = $$;
            // => additive_expression1
            $$ = _parse_additive_expression1();
            // <= additive_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // additive_operator
              final $1 = seq[0];
              // multiplicative_expression
              final $2 = seq[1];
              // additive_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= additive_operator multiplicative_expression additive_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '+', null
      _failure(_expect6);
    }
    // <= additive_operator multiplicative_expression additive_expression1 / '' # Choice
    if (_cacheable[21]) {
      _addToCache($$, pos, 21);
    }  
    return $$;
  }
  
  dynamic _parse_additive_operator() {
    // LEXEME
    // additive_operator <- plus / minus
    var $$;
    // => plus / minus # Choice
    switch (_ch == 43 ? 0 : _ch == -1 ? 2 : 1) {
      // [+]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => plus
          $$ = _parse_plus();
          // <= plus
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => minus
          $$ = _parse_minus();
          // <= minus
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '+'
      _failure(_expect18);
    }
    // <= plus / minus # Choice
    return $$;
  }
  
  dynamic _parse_ampersand() {
    // LEXEME
    // ampersand <- '&' spaces
    var $$;
    // => '&' spaces # Choice
    switch (_ch == 38 ? 0 : _ch == -1 ? 2 : 1) {
      // [&]
      case 0:
        // => '&' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '&'
          $$ = '&';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '&'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '&'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '&' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '&'
      _failure(_expect19);
    }
    // <= '&' spaces # Choice
    return $$;
  }
  
  dynamic _parse_ampersand2() {
    // LEXEME
    // ampersand2 <- '&&' spaces
    var $$;
    // => '&&' spaces # Choice
    switch (_ch == 38 ? 0 : _ch == -1 ? 2 : 1) {
      // [&]
      case 0:
        // => '&&' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '&&'
          $$ = _matchString(_strings0, '&&');
          // <= '&&'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '&&'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '&&' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '&&'
      _failure(_expect20);
    }
    // <= '&&' spaces # Choice
    return $$;
  }
  
  dynamic _parse_and_expression() {
    // NONTERMINAL
    // and_expression <- equality_expression and_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[11] >= pos) {
      $$ = _getFromCache(11);
    } else {
      _cachePos[11] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => equality_expression and_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => equality_expression and_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => equality_expression
          $$ = _parse_equality_expression();
          // <= equality_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => and_expression1
          $$ = _parse_and_expression1();
          // <= and_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // equality_expression
            final $1 = seq[0];
            // and_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= equality_expression and_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= equality_expression and_expression1 # Choice
    if (_cacheable[11]) {
      _addToCache($$, pos, 11);
    }  
    return $$;
  }
  
  dynamic _parse_and_expression1() {
    // NONTERMINAL
    // and_expression1 <- ampersand equality_expression and_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[25] >= pos) {
      $$ = _getFromCache(25);
    } else {
      _cachePos[25] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => ampersand equality_expression and_expression1 / '' # Choice
    switch (_getState(_transitions9)) {
      // [\u0000-%] [\'-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [&]
      case 1:
        while (true) {
          // => ampersand equality_expression and_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ampersand
            $$ = _parse_ampersand();
            // <= ampersand
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => equality_expression
            $$ = _parse_equality_expression();
            // <= equality_expression
            if (!success) break;
            seq[1] = $$;
            // => and_expression1
            $$ = _parse_and_expression1();
            // <= and_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // ampersand
              final $1 = seq[0];
              // equality_expression
              final $2 = seq[1];
              // and_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= ampersand equality_expression and_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '&', null
      _failure(_expect10);
    }
    // <= ampersand equality_expression and_expression1 / '' # Choice
    if (_cacheable[25]) {
      _addToCache($$, pos, 25);
    }  
    return $$;
  }
  
  dynamic _parse_any_char_until_eol() {
    // MORPHEME
    // any_char_until_eol <- !new_line .
    var $$;
    _beginToken(5);  
    // => !new_line . # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        // => !new_line . # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => !new_line
          var ch1 = _ch, pos1 = _cursor, testing0 = _testing; 
          _testing = _inputLen + 1;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          _ch = ch1;
          _cursor = pos1; 
          _testing = testing0;
          $$ = null;
          success = !success;
          // <= !new_line
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => .
          $$ = _matchAny();
          // <= .
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // !new_line
            final $1 = seq[0];
            // .
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $2;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= !new_line . # Sequence
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: any_char_until_eol
      _failure(_expect44);
    }
    // <= !new_line . # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_any_chars_until_eol() {
    // MORPHEME
    // any_chars_until_eol <- any_char_until_eol*
    var $$;
    _beginToken(6);  
    // => any_char_until_eol* # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => any_char_until_eol*
        var testing0 = _testing; 
        for (var reps = []; ; ) {
          _testing = _cursor;
          // => any_char_until_eol
          $$ = _parse_any_char_until_eol();
          // <= any_char_until_eol
          if (success) {  
            reps.add($$);
          } else {
            success = true;
            _testing = testing0;
            $$ = reps;
            break; 
          }
        }
        // <= any_char_until_eol*
        if (success) {    
          // any_char_until_eol*
          final $1 = $$;
          final $start = startPos0;
          $$ = _text();
        }
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = true;
        break;
      // EOF
      case 2:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => any_char_until_eol*
          var testing1 = _testing; 
          for (var reps = []; ; ) {
            _testing = _cursor;
            // => any_char_until_eol
            $$ = _parse_any_char_until_eol();
            // <= any_char_until_eol
            if (success) {  
              reps.add($$);
            } else {
              success = true;
              _testing = testing1;
              $$ = reps;
              break; 
            }
          }
          // <= any_char_until_eol*
          if (success) {    
            // any_char_until_eol*
            final $1 = $$;
            final $start = startPos1;
            $$ = _text();
          }
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => any_char_until_eol*
          var testing2 = _testing; 
          for (var reps = []; ; ) {
            _testing = _cursor;
            // => any_char_until_eol
            $$ = _parse_any_char_until_eol();
            // <= any_char_until_eol
            if (success) {  
              reps.add($$);
            } else {
              success = true;
              _testing = testing2;
              $$ = reps;
              break; 
            }
          }
          // <= any_char_until_eol*
          if (success) {    
            // any_char_until_eol*
            final $1 = $$;
            final $start = startPos2;
            $$ = _text();
          }
          _startPos = startPos2;
          break;
        }
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: any_chars_until_eol
      _failure(_expect45);
    }
    // <= any_char_until_eol* # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_asterisk() {
    // MORPHEME
    // asterisk <- '*' spaces
    var $$;
    _beginToken(7);  
    // => '*' spaces # Choice
    switch (_ch == 42 ? 0 : _ch == -1 ? 2 : 1) {
      // [*]
      case 0:
        // => '*' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '*'
          $$ = '*';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '*'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '*'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '*' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '*'
      _failure(_expect46);
    }
    // <= '*' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_binary_exponent_part() {
    // MORPHEME
    // binary_exponent_part <- ('p' / 'P') (sign / '') digit_sequence
    var $$;
    _beginToken(8);  
    // => ('p' / 'P') (sign / '') digit_sequence # Choice
    switch (_getState(_transitions19)) {
      // [P] [p]
      case 0:
        // => ('p' / 'P') (sign / '') digit_sequence # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ('p' / 'P') # Choice
          switch (_getState(_transitions20)) {
            // [P]
            case 0:
              var startPos1 = _startPos;
              _startPos = _cursor;
              // => 'P'
              $$ = 'P';
              success = true;
              if (++_cursor < _inputLen) {
                _ch = _input[_cursor];
              } else {
                _ch = -1;
              }
              // <= 'P'
              _startPos = startPos1;
              break;
            // [p]
            case 1:
              var startPos2 = _startPos;
              _startPos = _cursor;
              // => 'p'
              $$ = 'p';
              success = true;
              if (++_cursor < _inputLen) {
                _ch = _input[_cursor];
              } else {
                _ch = -1;
              }
              // <= 'p'
              _startPos = startPos2;
              break;
            // No matches
            // EOF
            case 2:
            case 3:
              $$ = null;
              success = false;
              break;
          }
          if (!success && _cursor > _testing) {
            // Expected: p, P
            _failure(_expect48);
          }
          // <= ('p' / 'P') # Choice
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => (sign / '') # Choice
          switch (_getState(_transitions21)) {
            // [\u0000-*] [,] [.-\u0010ffff]
            // EOF
            case 0:
            case 3:
              var startPos3 = _startPos;
              _startPos = _cursor;
              // => ''
              success = true;
              $$ = '';
              // <= ''
              _startPos = startPos3;
              break;
            // [+] [-]
            case 1:
              while (true) {
                var startPos4 = _startPos;
                _startPos = _cursor;
                // => sign
                $$ = _parse_sign();
                // <= sign
                _startPos = startPos4;
                if (success) break;
                var startPos5 = _startPos;
                _startPos = _cursor;
                // => ''
                success = true;
                $$ = '';
                // <= ''
                _startPos = startPos5;
                break;
              }
              break;
            // No matches
            case 2:
              $$ = null;
              success = false;
              break;
          }
          if (!success && _cursor > _testing) {
            // Expected: sign, null
            _failure(_expect49);
          }
          // <= (sign / '') # Choice
          if (!success) break;
          seq[1] = $$;
          // => digit_sequence
          $$ = _parse_digit_sequence();
          // <= digit_sequence
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // ('p' / 'P')
            final $1 = seq[0];
            // (sign / '')
            final $2 = seq[1];
            // digit_sequence
            final $3 = seq[2];
            final $start = startPos0;
            $$ = _text();
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= ('p' / 'P') (sign / '') digit_sequence # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: binary_exponent_part
      _failure(_expect47);
    }
    // <= ('p' / 'P') (sign / '') digit_sequence # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_c_char() {
    // MORPHEME
    // c_char <- ![\n\r'\\] . / escape_sequence
    var $$;
    _beginToken(9);  
    // => ![\n\r'\\] . / escape_sequence # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      case 0:
        while (true) {
          // => ![\n\r'\\] . # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ![\n\r'\\]
            var ch1 = _ch, pos1 = _cursor, testing0 = _testing; 
            _testing = _inputLen + 1;
            // => [\n\r'\\]
            $$ = _matchMapping(10, 92, _mapping1);
            // <= [\n\r'\\]
            _ch = ch1;
            _cursor = pos1; 
            _testing = testing0;
            $$ = null;
            success = !success;
            // <= ![\n\r'\\]
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => .
            $$ = _matchAny();
            // <= .
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // ![\n\r'\\]
              final $1 = seq[0];
              // .
              final $2 = seq[1];
              final $start = startPos0;
              $$ = $2.codeUnitAt(0);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= ![\n\r'\\] . # Sequence
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => escape_sequence
          $$ = _parse_escape_sequence();
          // <= escape_sequence
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
      // EOF
      case 2:
        // => ![\n\r'\\] . # Sequence
        var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ![\n\r'\\]
          var ch3 = _ch, pos3 = _cursor, testing1 = _testing; 
          _testing = _inputLen + 1;
          // => [\n\r'\\]
          $$ = _matchMapping(10, 92, _mapping1);
          // <= [\n\r'\\]
          _ch = ch3;
          _cursor = pos3; 
          _testing = testing1;
          $$ = null;
          success = !success;
          // <= ![\n\r'\\]
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => .
          $$ = _matchAny();
          // <= .
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // ![\n\r'\\]
            final $1 = seq[0];
            // .
            final $2 = seq[1];
            final $start = startPos2;
            $$ = $2.codeUnitAt(0);
          }
          break;
        }
        if (!success) {
          _ch = ch2;
          _cursor = pos2;
        }
        _startPos = startPos2;
        // <= ![\n\r'\\] . # Sequence
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: c_char
      _failure(_expect50);
    }
    // <= ![\n\r'\\] . / escape_sequence # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_character_constant_base() {
    // MORPHEME
    // character_constant_base <- '\'' c_char '\'' / 'L\'' c_char '\''
    var $$;
    _beginToken(10);  
    // => '\'' c_char '\'' / 'L\'' c_char '\'' # Choice
    switch (_getState(_transitions22)) {
      // [\']
      case 0:
        // => '\'' c_char '\'' # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '\''
          $$ = '\'';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '\''
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => c_char
          $$ = _parse_c_char();
          // <= c_char
          if (!success) break;
          seq[1] = $$;
          // => '\''
          $$ = _matchChar(39, '\'');
          // <= '\''
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // '\''
            final $1 = seq[0];
            // c_char
            final $2 = seq[1];
            // '\''
            final $3 = seq[2];
            final $start = startPos0;
            $$ = new CharacterLiteral(position: $start, text: _text(), value: $2);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '\'' c_char '\'' # Sequence
        break;
      // [L]
      case 1:
        // => 'L\'' c_char '\'' # Sequence
        var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => 'L\''
          $$ = _matchString(_strings12, 'L\'');
          // <= 'L\''
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => c_char
          $$ = _parse_c_char();
          // <= c_char
          if (!success) break;
          seq[1] = $$;
          // => '\''
          $$ = _matchChar(39, '\'');
          // <= '\''
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // 'L\''
            final $1 = seq[0];
            // c_char
            final $2 = seq[1];
            // '\''
            final $3 = seq[2];
            final $start = startPos1;
            $$ = new CharacterLiteral(position: $start, text: _text(), value: $2);
          }
          break;
        }
        if (!success) {
          _ch = ch1;
          _cursor = pos1;
        }
        _startPos = startPos1;
        // <= 'L\'' c_char '\'' # Sequence
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: character_constant_base
      _failure(_expect51);
    }
    // <= '\'' c_char '\'' / 'L\'' c_char '\'' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_conditional_expression() {
    // NONTERMINAL
    // conditional_expression <- logical_or_expression question_mark expression semicolon conditional_expression / logical_or_expression
    var $$;          
    var pos = _cursor;    
    if(_cachePos[6] >= pos) {
      $$ = _getFromCache(6);
    } else {
      _cachePos[6] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => logical_or_expression question_mark expression semicolon conditional_expression / logical_or_expression # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        while (true) {
          // => logical_or_expression question_mark expression semicolon conditional_expression # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => logical_or_expression
            $$ = _parse_logical_or_expression();
            // <= logical_or_expression
            if (!success) break;
            var seq = new List(5)..[0] = $$;
            // => question_mark
            $$ = _parse_question_mark();
            // <= question_mark
            if (!success) break;
            seq[1] = $$;
            // => expression
            $$ = _parse_expression();
            // <= expression
            if (!success) break;
            seq[2] = $$;
            // => semicolon
            $$ = _parse_semicolon();
            // <= semicolon
            if (!success) break;
            seq[3] = $$;
            // => conditional_expression
            $$ = _parse_conditional_expression();
            // <= conditional_expression
            if (!success) break;
            seq[4] = $$;
            $$ = seq;
            if (success) {    
              // logical_or_expression
              final $1 = seq[0];
              // question_mark
              final $2 = seq[1];
              // expression
              final $3 = seq[2];
              // semicolon
              final $4 = seq[3];
              // conditional_expression
              final $5 = seq[4];
              final $start = startPos0;
              $$ = _conditional($1, $3, $5, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= logical_or_expression question_mark expression semicolon conditional_expression # Sequence
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => logical_or_expression
          $$ = _parse_logical_or_expression();
          // <= logical_or_expression
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= logical_or_expression question_mark expression semicolon conditional_expression / logical_or_expression # Choice
    if (_cacheable[6]) {
      _addToCache($$, pos, 6);
    }  
    return $$;
  }
  
  dynamic _parse_constant() {
    // LEXEME
    // constant <- constant_base spaces
    var $$;
    // => constant_base spaces # Choice
    switch (_getState(_transitions13)) {
      // [\'] [.] [0-9] [L]
      case 0:
        // => constant_base spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => constant_base
          $$ = _parse_constant_base();
          // <= constant_base
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // constant_base
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= constant_base spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: constant
      _failure(_expect21);
    }
    // <= constant_base spaces # Choice
    return $$;
  }
  
  dynamic _parse_constant_base() {
    // MORPHEME
    // constant_base <- floating_constant_base / integer_constant_base / character_constant_base
    var $$;
    _beginToken(11);  
    // => floating_constant_base / integer_constant_base / character_constant_base # Choice
    switch (_getState(_transitions23)) {
      // [\'] [L]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => character_constant_base
        $$ = _parse_character_constant_base();
        // <= character_constant_base
        _startPos = startPos0;
        break;
      // [.]
      case 1:
        var startPos1 = _startPos;
        _startPos = _cursor;
        // => floating_constant_base
        $$ = _parse_floating_constant_base();
        // <= floating_constant_base
        _startPos = startPos1;
        break;
      // [0-9]
      case 2:
        while (true) {
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => floating_constant_base
          $$ = _parse_floating_constant_base();
          // <= floating_constant_base
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => integer_constant_base
          $$ = _parse_integer_constant_base();
          // <= integer_constant_base
          _startPos = startPos3;
          break;
        }
        break;
      // No matches
      // EOF
      case 3:
      case 4:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: constant_base
      _failure(_expect52);
    }
    // <= floating_constant_base / integer_constant_base / character_constant_base # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_constant_expression() {
    // NONTERMINAL
    // constant_expression <- conditional_expression
    var $$;          
    var pos = _cursor;    
    if(_cachePos[5] >= pos) {
      $$ = _getFromCache(5);
    } else {
      _cachePos[5] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => conditional_expression # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => conditional_expression
        $$ = _parse_conditional_expression();
        // <= conditional_expression
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= conditional_expression # Choice
    if (_cacheable[5]) {
      _addToCache($$, pos, 5);
    }  
    return $$;
  }
  
  dynamic _parse_control_line() {
    // LEXEME
    // control_line <- define_directive / undef_directive / error_directive
    var $$;
    // => define_directive / undef_directive / error_directive # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => define_directive
          $$ = _parse_define_directive();
          // <= define_directive
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => undef_directive
          $$ = _parse_undef_directive();
          // <= undef_directive
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => error_directive
          $$ = _parse_error_directive();
          // <= error_directive
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: control_line
      _failure(_expect22);
    }
    // <= define_directive / undef_directive / error_directive # Choice
    return $$;
  }
  
  dynamic _parse_decimal_constant() {
    // MORPHEME
    // decimal_constant <- nonzero_digit decimal_constant1
    var $$;
    _beginToken(12);  
    // => nonzero_digit decimal_constant1 # Choice
    switch (_ch >= 49 && _ch <= 57 ? 0 : _ch == -1 ? 2 : 1) {
      // [1-9]
      case 0:
        // => nonzero_digit decimal_constant1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => nonzero_digit
          $$ = _parse_nonzero_digit();
          // <= nonzero_digit
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => decimal_constant1
          $$ = _parse_decimal_constant1();
          // <= decimal_constant1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // nonzero_digit
            final $1 = seq[0];
            // decimal_constant1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = int.parse(_text());
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= nonzero_digit decimal_constant1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: decimal_constant
      _failure(_expect53);
    }
    // <= nonzero_digit decimal_constant1 # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_decimal_constant1() {
    // MORPHEME
    // decimal_constant1 <- digit decimal_constant1 / ''
    var $$;
    _beginToken(13);  
    // => digit decimal_constant1 / '' # Choice
    switch (_getState(_transitions24)) {
      // [\u0000-/] [:-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        _startPos = startPos0;
        break;
      // [0-9]
      case 1:
        while (true) {
          // => digit decimal_constant1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => digit
            $$ = _parse_digit();
            // <= digit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => decimal_constant1
            $$ = _parse_decimal_constant1();
            // <= decimal_constant1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= digit decimal_constant1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: decimal_constant1
      _failure(_expect54);
    }
    // <= digit decimal_constant1 / '' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_decimal_floating_constant() {
    // MORPHEME
    // decimal_floating_constant <- decimal_floating_constant_base spaces
    var $$;
    _beginToken(14);  
    // => decimal_floating_constant_base spaces # Choice
    switch (_getState(_transitions25)) {
      // [.] [0-9]
      case 0:
        // => decimal_floating_constant_base spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => decimal_floating_constant_base
          $$ = _parse_decimal_floating_constant_base();
          // <= decimal_floating_constant_base
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // decimal_floating_constant_base
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= decimal_floating_constant_base spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: decimal_floating_constant
      _failure(_expect55);
    }
    // <= decimal_floating_constant_base spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_decimal_floating_constant_base() {
    // MORPHEME
    // decimal_floating_constant_base <- fractional_constant exponent_part? floating_suffix? / digit_sequence exponent_part floating_suffix?
    var $$;
    _beginToken(15);  
    // => fractional_constant exponent_part? floating_suffix? / digit_sequence exponent_part floating_suffix? # Choice
    switch (_getState(_transitions26)) {
      // [.]
      case 0:
        // => fractional_constant exponent_part? floating_suffix? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => fractional_constant
          $$ = _parse_fractional_constant();
          // <= fractional_constant
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => exponent_part?
          var testing0 = _testing;
          _testing = _cursor;
          // => exponent_part
          $$ = _parse_exponent_part();
          // <= exponent_part
          success = true; 
          _testing = testing0;
          // <= exponent_part?
          if (!success) break;
          seq[1] = $$;
          // => floating_suffix?
          var testing1 = _testing;
          _testing = _cursor;
          // => floating_suffix
          $$ = _parse_floating_suffix();
          // <= floating_suffix
          success = true; 
          _testing = testing1;
          // <= floating_suffix?
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // fractional_constant
            final $1 = seq[0];
            // exponent_part?
            final $2 = seq[1];
            // floating_suffix?
            final $3 = seq[2];
            final $start = startPos0;
            $$ = _floatLiteral(_text(), $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= fractional_constant exponent_part? floating_suffix? # Sequence
        break;
      // [0-9]
      case 1:
        while (true) {
          // => fractional_constant exponent_part? floating_suffix? # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => fractional_constant
            $$ = _parse_fractional_constant();
            // <= fractional_constant
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => exponent_part?
            var testing2 = _testing;
            _testing = _cursor;
            // => exponent_part
            $$ = _parse_exponent_part();
            // <= exponent_part
            success = true; 
            _testing = testing2;
            // <= exponent_part?
            if (!success) break;
            seq[1] = $$;
            // => floating_suffix?
            var testing3 = _testing;
            _testing = _cursor;
            // => floating_suffix
            $$ = _parse_floating_suffix();
            // <= floating_suffix
            success = true; 
            _testing = testing3;
            // <= floating_suffix?
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // fractional_constant
              final $1 = seq[0];
              // exponent_part?
              final $2 = seq[1];
              // floating_suffix?
              final $3 = seq[2];
              final $start = startPos1;
              $$ = _floatLiteral(_text(), $start);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= fractional_constant exponent_part? floating_suffix? # Sequence
          if (success) break;
          // => digit_sequence exponent_part floating_suffix? # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => digit_sequence
            $$ = _parse_digit_sequence();
            // <= digit_sequence
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => exponent_part
            $$ = _parse_exponent_part();
            // <= exponent_part
            if (!success) break;
            seq[1] = $$;
            // => floating_suffix?
            var testing4 = _testing;
            _testing = _cursor;
            // => floating_suffix
            $$ = _parse_floating_suffix();
            // <= floating_suffix
            success = true; 
            _testing = testing4;
            // <= floating_suffix?
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // digit_sequence
              final $1 = seq[0];
              // exponent_part
              final $2 = seq[1];
              // floating_suffix?
              final $3 = seq[2];
              final $start = startPos2;
              $$ = _floatLiteral(_text(), $start);
            }
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= digit_sequence exponent_part floating_suffix? # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: decimal_floating_constant_base
      _failure(_expect56);
    }
    // <= fractional_constant exponent_part? floating_suffix? / digit_sequence exponent_part floating_suffix? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_define() {
    // MORPHEME
    // define <- '#define' spaces
    var $$;
    _beginToken(16);  
    // => '#define' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#define' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#define'
          $$ = _matchString(_strings13, '#define');
          // <= '#define'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#define'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#define' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#define'
      _failure(_expect57);
    }
    // <= '#define' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_define_directive() {
    // MORPHEME
    // define_directive <- define identifier replacement_list new_line?
    var $$;
    _beginToken(17);  
    // => define identifier replacement_list new_line? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => define identifier replacement_list new_line? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => define
          $$ = _parse_define();
          // <= define
          if (!success) break;
          var seq = new List(4)..[0] = $$;
          // => identifier
          $$ = _parse_identifier();
          // <= identifier
          if (!success) break;
          seq[1] = $$;
          // => replacement_list
          $$ = _parse_replacement_list();
          // <= replacement_list
          if (!success) break;
          seq[2] = $$;
          // => new_line?
          var testing0 = _testing;
          _testing = _cursor;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          success = true; 
          _testing = testing0;
          // <= new_line?
          if (!success) break;
          seq[3] = $$;
          $$ = seq;
          if (success) {    
            // define
            final $1 = seq[0];
            // identifier
            final $2 = seq[1];
            // replacement_list
            final $3 = seq[2];
            // new_line?
            final $4 = seq[3];
            final $start = startPos0;
            $$ = new DefineDirective(identifier: $2, replacement: $3, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= define identifier replacement_list new_line? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#define'
      _failure(_expect57);
    }
    // <= define identifier replacement_list new_line? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_defined() {
    // LEXEME
    // defined <- DEFINED lparen identifier rparen
    var $$;
    // => DEFINED lparen identifier rparen # Choice
    switch (_ch == 100 ? 0 : _ch == -1 ? 2 : 1) {
      // [d]
      case 0:
        // => DEFINED lparen identifier rparen # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => DEFINED
          $$ = _parse_DEFINED();
          // <= DEFINED
          if (!success) break;
          var seq = new List(4)..[0] = $$;
          // => lparen
          $$ = _parse_lparen();
          // <= lparen
          if (!success) break;
          seq[1] = $$;
          // => identifier
          $$ = _parse_identifier();
          // <= identifier
          if (!success) break;
          seq[2] = $$;
          // => rparen
          $$ = _parse_rparen();
          // <= rparen
          if (!success) break;
          seq[3] = $$;
          $$ = seq;
          if (success) {    
            // DEFINED
            final $1 = seq[0];
            // lparen
            final $2 = seq[1];
            // identifier
            final $3 = seq[2];
            // rparen
            final $4 = seq[3];
            final $start = startPos0;
            $$ = new DefinedExpression(identifier: $3, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= DEFINED lparen identifier rparen # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined'
      _failure(_expect23);
    }
    // <= DEFINED lparen identifier rparen # Choice
    return $$;
  }
  
  dynamic _parse_digit() {
    // MORPHEME
    // digit <- [0-9]
    var $$;
    _beginToken(18);  
    // => [0-9] # Choice
    switch (_ch >= 48 && _ch <= 57 ? 0 : _ch == -1 ? 2 : 1) {
      // [0-9]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [0-9]
        $$ = _matchRange(48, 57);
        // <= [0-9]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: digit
      _failure(_expect58);
    }
    // <= [0-9] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_digit_sequence() {
    // MORPHEME
    // digit_sequence <- digit+
    var $$;
    _beginToken(19);  
    // => digit+ # Choice
    switch (_ch >= 48 && _ch <= 57 ? 0 : _ch == -1 ? 2 : 1) {
      // [0-9]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => digit+
        var testing0;
        for (var first = true, reps; ;) {  
          // => digit  
          $$ = _parse_digit();  
          // <= digit  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= digit+
        if (success) {    
          // digit+
          final $1 = $$;
          final $start = startPos0;
          $$ = _text();
        }
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: digit_sequence
      _failure(_expect59);
    }
    // <= digit+ # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_elif() {
    // LEXEME
    // elif <- '#elif' spaces
    var $$;
    // => '#elif' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#elif' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#elif'
          $$ = _matchString(_strings1, '#elif');
          // <= '#elif'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#elif'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#elif' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#elif'
      _failure(_expect15);
    }
    // <= '#elif' spaces # Choice
    return $$;
  }
  
  dynamic _parse_elif_group() {
    // NONTERMINAL
    // elif_group <- elif constant_expression new_line group?
    var $$;
    // => elif constant_expression new_line group? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => elif constant_expression new_line group? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => elif
          $$ = _parse_elif();
          // <= elif
          if (!success) break;
          var seq = new List(4)..[0] = $$;
          // => constant_expression
          $$ = _parse_constant_expression();
          // <= constant_expression
          if (!success) break;
          seq[1] = $$;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          if (!success) break;
          seq[2] = $$;
          // => group?
          var testing0 = _testing;
          _testing = _cursor;
          // => group
          $$ = _parse_group();
          // <= group
          success = true; 
          _testing = testing0;
          // <= group?
          if (!success) break;
          seq[3] = $$;
          $$ = seq;
          if (success) {    
            // elif
            final $1 = seq[0];
            // constant_expression
            final $2 = seq[1];
            // new_line
            final $3 = seq[2];
            // group?
            final $4 = seq[3];
            final $start = startPos0;
            $$ = new IfDirective(body: $4, condition: $2, name: $1, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= elif constant_expression new_line group? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#elif'
      _failure(_expect15);
    }
    // <= elif constant_expression new_line group? # Choice
    return $$;
  }
  
  dynamic _parse_elif_groups() {
    // NONTERMINAL
    // elif_groups <- elif_group+
    var $$;
    // => elif_group+ # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => elif_group+
        var testing0;
        for (var first = true, reps; ;) {  
          // => elif_group  
          $$ = _parse_elif_group();  
          // <= elif_group  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= elif_group+
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#elif'
      _failure(_expect15);
    }
    // <= elif_group+ # Choice
    return $$;
  }
  
  dynamic _parse_else() {
    // LEXEME
    // else <- '#else' spaces
    var $$;
    // => '#else' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#else' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#else'
          $$ = _matchString(_strings2, '#else');
          // <= '#else'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#else'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#else' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#else'
      _failure(_expect16);
    }
    // <= '#else' spaces # Choice
    return $$;
  }
  
  dynamic _parse_else_group() {
    // NONTERMINAL
    // else_group <- else new_line group?
    var $$;
    // => else new_line group? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => else new_line group? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => else
          $$ = _parse_else();
          // <= else
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          if (!success) break;
          seq[1] = $$;
          // => group?
          var testing0 = _testing;
          _testing = _cursor;
          // => group
          $$ = _parse_group();
          // <= group
          success = true; 
          _testing = testing0;
          // <= group?
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // else
            final $1 = seq[0];
            // new_line
            final $2 = seq[1];
            // group?
            final $3 = seq[2];
            final $start = startPos0;
            $$ = new ElseDirective(body: $3, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= else new_line group? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#else'
      _failure(_expect16);
    }
    // <= else new_line group? # Choice
    return $$;
  }
  
  dynamic _parse_endif_line() {
    // LEXEME
    // endif_line <- '#endif' new_line?
    var $$;
    // => '#endif' new_line? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#endif' new_line? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#endif'
          $$ = _matchString(_strings3, '#endif');
          // <= '#endif'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => new_line?
          var testing0 = _testing;
          _testing = _cursor;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          success = true; 
          _testing = testing0;
          // <= new_line?
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#endif'
            final $1 = seq[0];
            // new_line?
            final $2 = seq[1];
            final $start = startPos0;
            $$ = new EndifDirective(position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#endif' new_line? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#endif'
      _failure(_expect24);
    }
    // <= '#endif' new_line? # Choice
    return $$;
  }
  
  dynamic _parse_eof() {
    // LEXEME
    // eof <- !.
    var $$;          
    var pos = _cursor;    
    if(_cachePos[43] >= pos) {
      $$ = _getFromCache(43);
    } else {
      _cachePos[43] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => !. # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => !.
        var ch0 = _ch, pos0 = _cursor, testing0 = _testing; 
        _testing = _inputLen + 1;
        // => .
        $$ = _matchAny();
        // <= .
        _ch = ch0;
        _cursor = pos0; 
        _testing = testing0;
        $$ = null;
        success = !success;
        // <= !.
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: eof
      _failure(_expect0);
    }
    // <= !. # Choice
    if (_cacheable[43]) {
      _addToCache($$, pos, 43);
    }  
    return $$;
  }
  
  dynamic _parse_eq() {
    // LEXEME
    // eq <- '==' spaces
    var $$;
    // => '==' spaces # Choice
    switch (_ch == 61 ? 0 : _ch == -1 ? 2 : 1) {
      // [=]
      case 0:
        // => '==' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '=='
          $$ = _matchString(_strings4, '==');
          // <= '=='
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '=='
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '==' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '=='
      _failure(_expect25);
    }
    // <= '==' spaces # Choice
    return $$;
  }
  
  dynamic _parse_equality_expression() {
    // NONTERMINAL
    // equality_expression <- relational_expression equality_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[12] >= pos) {
      $$ = _getFromCache(12);
    } else {
      _cachePos[12] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => relational_expression equality_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => relational_expression equality_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => relational_expression
          $$ = _parse_relational_expression();
          // <= relational_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => equality_expression1
          $$ = _parse_equality_expression1();
          // <= equality_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // relational_expression
            final $1 = seq[0];
            // equality_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= relational_expression equality_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= relational_expression equality_expression1 # Choice
    if (_cacheable[12]) {
      _addToCache($$, pos, 12);
    }  
    return $$;
  }
  
  dynamic _parse_equality_expression1() {
    // NONTERMINAL
    // equality_expression1 <- eq relational_expression equality_expression1 / neq relational_expression equality_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[24] >= pos) {
      $$ = _getFromCache(24);
    } else {
      _cachePos[24] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => eq relational_expression equality_expression1 / neq relational_expression equality_expression1 / '' # Choice
    switch (_getState(_transitions8)) {
      // [\u0000- ] [\"-<] [>-\u0010ffff]
      // EOF
      case 0:
      case 4:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [!]
      case 1:
        while (true) {
          // => neq relational_expression equality_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => neq
            $$ = _parse_neq();
            // <= neq
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => relational_expression
            $$ = _parse_relational_expression();
            // <= relational_expression
            if (!success) break;
            seq[1] = $$;
            // => equality_expression1
            $$ = _parse_equality_expression1();
            // <= equality_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // neq
              final $1 = seq[0];
              // relational_expression
              final $2 = seq[1];
              // equality_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= neq relational_expression equality_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // [=]
      case 2:
        while (true) {
          // => eq relational_expression equality_expression1 # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos3 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => eq
            $$ = _parse_eq();
            // <= eq
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => relational_expression
            $$ = _parse_relational_expression();
            // <= relational_expression
            if (!success) break;
            seq[1] = $$;
            // => equality_expression1
            $$ = _parse_equality_expression1();
            // <= equality_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // eq
              final $1 = seq[0];
              // relational_expression
              final $2 = seq[1];
              // equality_expression1
              final $3 = seq[2];
              final $start = startPos3;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos3;
          // <= eq relational_expression equality_expression1 # Sequence
          if (success) break;
          var startPos4 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos4;
            $$ = null;
          }
          _startPos = startPos4;
          break;
        }
        break;
      // No matches
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '==', '!=', null
      _failure(_expect9);
    }
    // <= eq relational_expression equality_expression1 / neq relational_expression equality_expression1 / '' # Choice
    if (_cacheable[24]) {
      _addToCache($$, pos, 24);
    }  
    return $$;
  }
  
  dynamic _parse_error() {
    // MORPHEME
    // error <- '#error' spaces
    var $$;
    _beginToken(20);  
    // => '#error' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#error' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#error'
          $$ = _matchString(_strings14, '#error');
          // <= '#error'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#error'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#error' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#error'
      _failure(_expect60);
    }
    // <= '#error' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_error_directive() {
    // MORPHEME
    // error_directive <- error any_chars_until_eol new_line?
    var $$;
    _beginToken(21);  
    // => error any_chars_until_eol new_line? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => error any_chars_until_eol new_line? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => error
          $$ = _parse_error();
          // <= error
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => any_chars_until_eol
          $$ = _parse_any_chars_until_eol();
          // <= any_chars_until_eol
          if (!success) break;
          seq[1] = $$;
          // => new_line?
          var testing0 = _testing;
          _testing = _cursor;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          success = true; 
          _testing = testing0;
          // <= new_line?
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // error
            final $1 = seq[0];
            // any_chars_until_eol
            final $2 = seq[1];
            // new_line?
            final $3 = seq[2];
            final $start = startPos0;
            $$ = new ErrorDirective(message: $2, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= error any_chars_until_eol new_line? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#error'
      _failure(_expect60);
    }
    // <= error any_chars_until_eol new_line? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_escape_sequence() {
    // MORPHEME
    // escape_sequence <- simple_escape_sequence / octal_escape_sequence / hexadecimal_escape_sequence / universal_character_name
    var $$;
    _beginToken(22);  
    // => simple_escape_sequence / octal_escape_sequence / hexadecimal_escape_sequence / universal_character_name # Choice
    switch (_ch == 92 ? 0 : _ch == -1 ? 2 : 1) {
      // [\\]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => simple_escape_sequence
          $$ = _parse_simple_escape_sequence();
          // <= simple_escape_sequence
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => octal_escape_sequence
          $$ = _parse_octal_escape_sequence();
          // <= octal_escape_sequence
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => hexadecimal_escape_sequence
          $$ = _parse_hexadecimal_escape_sequence();
          // <= hexadecimal_escape_sequence
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => universal_character_name
          $$ = _parse_universal_character_name();
          // <= universal_character_name
          _startPos = startPos3;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: escape_sequence
      _failure(_expect61);
    }
    // <= simple_escape_sequence / octal_escape_sequence / hexadecimal_escape_sequence / universal_character_name # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_exclamation() {
    // MORPHEME
    // exclamation <- '!' spaces
    var $$;
    _beginToken(23);  
    // => '!' spaces # Choice
    switch (_ch == 33 ? 0 : _ch == -1 ? 2 : 1) {
      // [!]
      case 0:
        // => '!' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '!'
          $$ = '!';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '!'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '!'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '!' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '!'
      _failure(_expect62);
    }
    // <= '!' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_exclusive_or_expression() {
    // NONTERMINAL
    // exclusive_or_expression <- and_expression exclusive_or_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[10] >= pos) {
      $$ = _getFromCache(10);
    } else {
      _cachePos[10] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => and_expression exclusive_or_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => and_expression exclusive_or_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => and_expression
          $$ = _parse_and_expression();
          // <= and_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => exclusive_or_expression1
          $$ = _parse_exclusive_or_expression1();
          // <= exclusive_or_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // and_expression
            final $1 = seq[0];
            // exclusive_or_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= and_expression exclusive_or_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= and_expression exclusive_or_expression1 # Choice
    if (_cacheable[10]) {
      _addToCache($$, pos, 10);
    }  
    return $$;
  }
  
  dynamic _parse_exclusive_or_expression1() {
    // NONTERMINAL
    // exclusive_or_expression1 <- xor and_expression exclusive_or_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[26] >= pos) {
      $$ = _getFromCache(26);
    } else {
      _cachePos[26] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => xor and_expression exclusive_or_expression1 / '' # Choice
    switch (_getState(_transitions10)) {
      // [\u0000-]] [_-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [^]
      case 1:
        while (true) {
          // => xor and_expression exclusive_or_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => xor
            $$ = _parse_xor();
            // <= xor
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => and_expression
            $$ = _parse_and_expression();
            // <= and_expression
            if (!success) break;
            seq[1] = $$;
            // => exclusive_or_expression1
            $$ = _parse_exclusive_or_expression1();
            // <= exclusive_or_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // xor
              final $1 = seq[0];
              // and_expression
              final $2 = seq[1];
              // exclusive_or_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= xor and_expression exclusive_or_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '^', null
      _failure(_expect11);
    }
    // <= xor and_expression exclusive_or_expression1 / '' # Choice
    if (_cacheable[26]) {
      _addToCache($$, pos, 26);
    }  
    return $$;
  }
  
  dynamic _parse_exponent_part() {
    // MORPHEME
    // exponent_part <- ('e' / 'E') (sign / '') digit_sequence
    var $$;
    _beginToken(24);  
    // => ('e' / 'E') (sign / '') digit_sequence # Choice
    switch (_getState(_transitions27)) {
      // [E] [e]
      case 0:
        // => ('e' / 'E') (sign / '') digit_sequence # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ('e' / 'E') # Choice
          switch (_getState(_transitions28)) {
            // [E]
            case 0:
              var startPos1 = _startPos;
              _startPos = _cursor;
              // => 'E'
              $$ = 'E';
              success = true;
              if (++_cursor < _inputLen) {
                _ch = _input[_cursor];
              } else {
                _ch = -1;
              }
              // <= 'E'
              _startPos = startPos1;
              break;
            // [e]
            case 1:
              var startPos2 = _startPos;
              _startPos = _cursor;
              // => 'e'
              $$ = 'e';
              success = true;
              if (++_cursor < _inputLen) {
                _ch = _input[_cursor];
              } else {
                _ch = -1;
              }
              // <= 'e'
              _startPos = startPos2;
              break;
            // No matches
            // EOF
            case 2:
            case 3:
              $$ = null;
              success = false;
              break;
          }
          if (!success && _cursor > _testing) {
            // Expected: e, E
            _failure(_expect64);
          }
          // <= ('e' / 'E') # Choice
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => (sign / '') # Choice
          switch (_getState(_transitions21)) {
            // [\u0000-*] [,] [.-\u0010ffff]
            // EOF
            case 0:
            case 3:
              var startPos3 = _startPos;
              _startPos = _cursor;
              // => ''
              success = true;
              $$ = '';
              // <= ''
              _startPos = startPos3;
              break;
            // [+] [-]
            case 1:
              while (true) {
                var startPos4 = _startPos;
                _startPos = _cursor;
                // => sign
                $$ = _parse_sign();
                // <= sign
                _startPos = startPos4;
                if (success) break;
                var startPos5 = _startPos;
                _startPos = _cursor;
                // => ''
                success = true;
                $$ = '';
                // <= ''
                _startPos = startPos5;
                break;
              }
              break;
            // No matches
            case 2:
              $$ = null;
              success = false;
              break;
          }
          if (!success && _cursor > _testing) {
            // Expected: sign, null
            _failure(_expect49);
          }
          // <= (sign / '') # Choice
          if (!success) break;
          seq[1] = $$;
          // => digit_sequence
          $$ = _parse_digit_sequence();
          // <= digit_sequence
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // ('e' / 'E')
            final $1 = seq[0];
            // (sign / '')
            final $2 = seq[1];
            // digit_sequence
            final $3 = seq[2];
            final $start = startPos0;
            $$ = _text();
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= ('e' / 'E') (sign / '') digit_sequence # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: exponent_part
      _failure(_expect63);
    }
    // <= ('e' / 'E') (sign / '') digit_sequence # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_expression() {
    // NONTERMINAL
    // expression <- conditional_expression
    var $$;          
    var pos = _cursor;    
    if(_cachePos[19] >= pos) {
      $$ = _getFromCache(19);
    } else {
      _cachePos[19] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => conditional_expression # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => conditional_expression
        $$ = _parse_conditional_expression();
        // <= conditional_expression
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= conditional_expression # Choice
    if (_cacheable[19]) {
      _addToCache($$, pos, 19);
    }  
    return $$;
  }
  
  dynamic _parse_floating_constant_base() {
    // MORPHEME
    // floating_constant_base <- decimal_floating_constant / hexadecimal_floating_constant
    var $$;
    _beginToken(25);  
    // => decimal_floating_constant / hexadecimal_floating_constant # Choice
    switch (_getState(_transitions26)) {
      // [.]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => decimal_floating_constant
        $$ = _parse_decimal_floating_constant();
        // <= decimal_floating_constant
        _startPos = startPos0;
        break;
      // [0-9]
      case 1:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => decimal_floating_constant
          $$ = _parse_decimal_floating_constant();
          // <= decimal_floating_constant
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => hexadecimal_floating_constant
          $$ = _parse_hexadecimal_floating_constant();
          // <= hexadecimal_floating_constant
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: floating_constant_base
      _failure(_expect65);
    }
    // <= decimal_floating_constant / hexadecimal_floating_constant # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_floating_suffix() {
    // MORPHEME
    // floating_suffix <- [FLfl]
    var $$;
    _beginToken(26);  
    // => [FLfl] # Choice
    switch (_getState(_transitions29)) {
      // [F] [L] [f] [l]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [FLfl]
        $$ = _matchMapping(70, 108, _mapping2);
        // <= [FLfl]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: floating_suffix
      _failure(_expect66);
    }
    // <= [FLfl] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_fractional_constant() {
    // MORPHEME
    // fractional_constant <- digit_sequence? '.' digit_sequence / digit_sequence '.'
    var $$;
    _beginToken(27);  
    // => digit_sequence? '.' digit_sequence / digit_sequence '.' # Choice
    switch (_getState(_transitions26)) {
      // [.]
      case 0:
        // => digit_sequence? '.' digit_sequence # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => digit_sequence?
          var testing0 = _testing;
          _testing = _cursor;
          // => digit_sequence
          $$ = _parse_digit_sequence();
          // <= digit_sequence
          success = true; 
          _testing = testing0;
          // <= digit_sequence?
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => '.'
          $$ = _matchChar(46, '.');
          // <= '.'
          if (!success) break;
          seq[1] = $$;
          // => digit_sequence
          $$ = _parse_digit_sequence();
          // <= digit_sequence
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // digit_sequence?
            final $1 = seq[0];
            // '.'
            final $2 = seq[1];
            // digit_sequence
            final $3 = seq[2];
            final $start = startPos0;
            $$ = _text();
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= digit_sequence? '.' digit_sequence # Sequence
        break;
      // [0-9]
      case 1:
        while (true) {
          // => digit_sequence? '.' digit_sequence # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => digit_sequence?
            var testing1 = _testing;
            _testing = _cursor;
            // => digit_sequence
            $$ = _parse_digit_sequence();
            // <= digit_sequence
            success = true; 
            _testing = testing1;
            // <= digit_sequence?
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => '.'
            $$ = _matchChar(46, '.');
            // <= '.'
            if (!success) break;
            seq[1] = $$;
            // => digit_sequence
            $$ = _parse_digit_sequence();
            // <= digit_sequence
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // digit_sequence?
              final $1 = seq[0];
              // '.'
              final $2 = seq[1];
              // digit_sequence
              final $3 = seq[2];
              final $start = startPos1;
              $$ = _text();
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= digit_sequence? '.' digit_sequence # Sequence
          if (success) break;
          // => digit_sequence '.' # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => digit_sequence
            $$ = _parse_digit_sequence();
            // <= digit_sequence
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => '.'
            $$ = _matchChar(46, '.');
            // <= '.'
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // digit_sequence
              final $1 = seq[0];
              // '.'
              final $2 = seq[1];
              final $start = startPos2;
              $$ = _text();
            }
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= digit_sequence '.' # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: fractional_constant
      _failure(_expect67);
    }
    // <= digit_sequence? '.' digit_sequence / digit_sequence '.' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_group() {
    // NONTERMINAL
    // group <- group_part+
    var $$;          
    var pos = _cursor;    
    if(_cachePos[1] >= pos) {
      $$ = _getFromCache(1);
    } else {
      _cachePos[1] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => group_part+ # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => group_part+
        var testing0;
        for (var first = true, reps; ;) {  
          // => group_part  
          $$ = _parse_group_part();  
          // <= group_part  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= group_part+
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#if', '#ifdef', '#ifndef', control_line, text_line
      _failure(_expect1);
    }
    // <= group_part+ # Choice
    if (_cacheable[1]) {
      _addToCache($$, pos, 1);
    }  
    return $$;
  }
  
  dynamic _parse_group_part() {
    // NONTERMINAL
    // group_part <- if_section / control_line / text_line
    var $$;
    // => if_section / control_line / text_line # Choice
    switch (_getState(_transitions0)) {
      // [\u0000-\"] [\$-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => text_line
        $$ = _parse_text_line();
        // <= text_line
        _startPos = startPos0;
        break;
      // [#]
      case 1:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => if_section
          $$ = _parse_if_section();
          // <= if_section
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => control_line
          $$ = _parse_control_line();
          // <= control_line
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => text_line
          $$ = _parse_text_line();
          // <= text_line
          _startPos = startPos3;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#if', '#ifdef', '#ifndef', control_line, text_line
      _failure(_expect1);
    }
    // <= if_section / control_line / text_line # Choice
    return $$;
  }
  
  dynamic _parse_gt() {
    // MORPHEME
    // gt <- '>' spaces
    var $$;
    _beginToken(28);  
    // => '>' spaces # Choice
    switch (_ch == 62 ? 0 : _ch == -1 ? 2 : 1) {
      // [>]
      case 0:
        // => '>' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '>'
          $$ = '>';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '>'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '>'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '>' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '>'
      _failure(_expect68);
    }
    // <= '>' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_gte() {
    // MORPHEME
    // gte <- '>=' spaces
    var $$;
    _beginToken(29);  
    // => '>=' spaces # Choice
    switch (_ch == 62 ? 0 : _ch == -1 ? 2 : 1) {
      // [>]
      case 0:
        // => '>=' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '>='
          $$ = _matchString(_strings15, '>=');
          // <= '>='
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '>='
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '>=' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '>='
      _failure(_expect69);
    }
    // <= '>=' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hex_quad() {
    // MORPHEME
    // hex_quad <- hexadecimal_digit hexadecimal_digit hexadecimal_digit hexadecimal_digit
    var $$;
    _beginToken(30);  
    // => hexadecimal_digit hexadecimal_digit hexadecimal_digit hexadecimal_digit # Choice
    switch (_getState(_transitions30)) {
      // [0-a] [f]
      case 0:
        // => hexadecimal_digit hexadecimal_digit hexadecimal_digit hexadecimal_digit # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          var seq = new List(4)..[0] = $$;
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          seq[1] = $$;
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          seq[2] = $$;
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          seq[3] = $$;
          $$ = seq;
          if (success) {    
            // hexadecimal_digit
            final $1 = seq[0];
            // hexadecimal_digit
            final $2 = seq[1];
            // hexadecimal_digit
            final $3 = seq[2];
            // hexadecimal_digit
            final $4 = seq[3];
            final $start = startPos0;
            $$ = _text();
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= hexadecimal_digit hexadecimal_digit hexadecimal_digit hexadecimal_digit # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hex_quad
      _failure(_expect70);
    }
    // <= hexadecimal_digit hexadecimal_digit hexadecimal_digit hexadecimal_digit # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_constant() {
    // MORPHEME
    // hexadecimal_constant <- hexadecimal_prefix hexadecimal_digit hexadecimal_constant1
    var $$;
    _beginToken(31);  
    // => hexadecimal_prefix hexadecimal_digit hexadecimal_constant1 # Choice
    switch (_ch == 48 ? 0 : _ch == -1 ? 2 : 1) {
      // [0]
      case 0:
        // => hexadecimal_prefix hexadecimal_digit hexadecimal_constant1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => hexadecimal_prefix
          $$ = _parse_hexadecimal_prefix();
          // <= hexadecimal_prefix
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          seq[1] = $$;
          // => hexadecimal_constant1
          $$ = _parse_hexadecimal_constant1();
          // <= hexadecimal_constant1
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // hexadecimal_prefix
            final $1 = seq[0];
            // hexadecimal_digit
            final $2 = seq[1];
            // hexadecimal_constant1
            final $3 = seq[2];
            final $start = startPos0;
            $$ = int.parse(_text(2), radix: 16);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= hexadecimal_prefix hexadecimal_digit hexadecimal_constant1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_constant
      _failure(_expect71);
    }
    // <= hexadecimal_prefix hexadecimal_digit hexadecimal_constant1 # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_constant1() {
    // MORPHEME
    // hexadecimal_constant1 <- hexadecimal_digit hexadecimal_constant1 / ''
    var $$;
    _beginToken(32);  
    // => hexadecimal_digit hexadecimal_constant1 / '' # Choice
    switch (_getState(_transitions31)) {
      // [\u0000-/] [b-e] [g-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        _startPos = startPos0;
        break;
      // [0-a] [f]
      case 1:
        while (true) {
          // => hexadecimal_digit hexadecimal_constant1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_digit
            $$ = _parse_hexadecimal_digit();
            // <= hexadecimal_digit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => hexadecimal_constant1
            $$ = _parse_hexadecimal_constant1();
            // <= hexadecimal_constant1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= hexadecimal_digit hexadecimal_constant1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_constant1
      _failure(_expect72);
    }
    // <= hexadecimal_digit hexadecimal_constant1 / '' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_digit() {
    // MORPHEME
    // hexadecimal_digit <- [0-af]
    var $$;
    _beginToken(33);  
    // => [0-af] # Choice
    switch (_getState(_transitions30)) {
      // [0-a] [f]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [0-af]
        $$ = _matchMapping(48, 102, _mapping3);
        // <= [0-af]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_digit
      _failure(_expect73);
    }
    // <= [0-af] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_digit_sequence() {
    // MORPHEME
    // hexadecimal_digit_sequence <- hexadecimal_digit+
    var $$;
    _beginToken(34);  
    // => hexadecimal_digit+ # Choice
    switch (_getState(_transitions30)) {
      // [0-a] [f]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => hexadecimal_digit+
        var testing0;
        for (var first = true, reps; ;) {  
          // => hexadecimal_digit  
          $$ = _parse_hexadecimal_digit();  
          // <= hexadecimal_digit  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= hexadecimal_digit+
        if (success) {    
          // hexadecimal_digit+
          final $1 = $$;
          final $start = startPos0;
          $$ = _text();
        }
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_digit_sequence
      _failure(_expect74);
    }
    // <= hexadecimal_digit+ # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_escape_sequence() {
    // MORPHEME
    // hexadecimal_escape_sequence <- '\\x' hexadecimal_digit hexadecimal_escape_sequence1
    var $$;
    _beginToken(35);  
    // => '\\x' hexadecimal_digit hexadecimal_escape_sequence1 # Choice
    switch (_ch == 92 ? 0 : _ch == -1 ? 2 : 1) {
      // [\\]
      case 0:
        // => '\\x' hexadecimal_digit hexadecimal_escape_sequence1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '\\x'
          $$ = _matchString(_strings16, '\\x');
          // <= '\\x'
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => hexadecimal_digit
          $$ = _parse_hexadecimal_digit();
          // <= hexadecimal_digit
          if (!success) break;
          seq[1] = $$;
          // => hexadecimal_escape_sequence1
          $$ = _parse_hexadecimal_escape_sequence1();
          // <= hexadecimal_escape_sequence1
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // '\\x'
            final $1 = seq[0];
            // hexadecimal_digit
            final $2 = seq[1];
            // hexadecimal_escape_sequence1
            final $3 = seq[2];
            final $start = startPos0;
            $$ = int.parse(_text(2), radix: 16);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '\\x' hexadecimal_digit hexadecimal_escape_sequence1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '\x'
      _failure(_expect75);
    }
    // <= '\\x' hexadecimal_digit hexadecimal_escape_sequence1 # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_escape_sequence1() {
    // MORPHEME
    // hexadecimal_escape_sequence1 <- hexadecimal_digit hexadecimal_escape_sequence1 / ''
    var $$;
    _beginToken(36);  
    // => hexadecimal_digit hexadecimal_escape_sequence1 / '' # Choice
    switch (_getState(_transitions31)) {
      // [\u0000-/] [b-e] [g-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        _startPos = startPos0;
        break;
      // [0-a] [f]
      case 1:
        while (true) {
          // => hexadecimal_digit hexadecimal_escape_sequence1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_digit
            $$ = _parse_hexadecimal_digit();
            // <= hexadecimal_digit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => hexadecimal_escape_sequence1
            $$ = _parse_hexadecimal_escape_sequence1();
            // <= hexadecimal_escape_sequence1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // hexadecimal_digit
              final $1 = seq[0];
              // hexadecimal_escape_sequence1
              final $2 = seq[1];
              final $start = startPos1;
              $$ = _text();
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= hexadecimal_digit hexadecimal_escape_sequence1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_escape_sequence1
      _failure(_expect76);
    }
    // <= hexadecimal_digit hexadecimal_escape_sequence1 / '' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_floating_constant() {
    // MORPHEME
    // hexadecimal_floating_constant <- hexadecimal_prefix hexadecimal_fractional_constant binary_exponent_part floating_suffix? / hexadecimal_prefix hexadecimal_digit_sequence binary_exponent_part floating_suffix?
    var $$;
    _beginToken(37);  
    // => hexadecimal_prefix hexadecimal_fractional_constant binary_exponent_part floating_suffix? / hexadecimal_prefix hexadecimal_digit_sequence binary_exponent_part floating_suffix? # Choice
    switch (_ch == 48 ? 0 : _ch == -1 ? 2 : 1) {
      // [0]
      case 0:
        while (true) {
          // => hexadecimal_prefix hexadecimal_fractional_constant binary_exponent_part floating_suffix? # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_prefix
            $$ = _parse_hexadecimal_prefix();
            // <= hexadecimal_prefix
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => hexadecimal_fractional_constant
            $$ = _parse_hexadecimal_fractional_constant();
            // <= hexadecimal_fractional_constant
            if (!success) break;
            seq[1] = $$;
            // => binary_exponent_part
            $$ = _parse_binary_exponent_part();
            // <= binary_exponent_part
            if (!success) break;
            seq[2] = $$;
            // => floating_suffix?
            var testing0 = _testing;
            _testing = _cursor;
            // => floating_suffix
            $$ = _parse_floating_suffix();
            // <= floating_suffix
            success = true; 
            _testing = testing0;
            // <= floating_suffix?
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // hexadecimal_prefix
              final $1 = seq[0];
              // hexadecimal_fractional_constant
              final $2 = seq[1];
              // binary_exponent_part
              final $3 = seq[2];
              // floating_suffix?
              final $4 = seq[3];
              final $start = startPos0;
              $$ = _floatLiteral(_text(), $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= hexadecimal_prefix hexadecimal_fractional_constant binary_exponent_part floating_suffix? # Sequence
          if (success) break;
          // => hexadecimal_prefix hexadecimal_digit_sequence binary_exponent_part floating_suffix? # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_prefix
            $$ = _parse_hexadecimal_prefix();
            // <= hexadecimal_prefix
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => hexadecimal_digit_sequence
            $$ = _parse_hexadecimal_digit_sequence();
            // <= hexadecimal_digit_sequence
            if (!success) break;
            seq[1] = $$;
            // => binary_exponent_part
            $$ = _parse_binary_exponent_part();
            // <= binary_exponent_part
            if (!success) break;
            seq[2] = $$;
            // => floating_suffix?
            var testing1 = _testing;
            _testing = _cursor;
            // => floating_suffix
            $$ = _parse_floating_suffix();
            // <= floating_suffix
            success = true; 
            _testing = testing1;
            // <= floating_suffix?
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // hexadecimal_prefix
              final $1 = seq[0];
              // hexadecimal_digit_sequence
              final $2 = seq[1];
              // binary_exponent_part
              final $3 = seq[2];
              // floating_suffix?
              final $4 = seq[3];
              final $start = startPos1;
              $$ = _floatLiteral(_text(), $start);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= hexadecimal_prefix hexadecimal_digit_sequence binary_exponent_part floating_suffix? # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_floating_constant
      _failure(_expect77);
    }
    // <= hexadecimal_prefix hexadecimal_fractional_constant binary_exponent_part floating_suffix? / hexadecimal_prefix hexadecimal_digit_sequence binary_exponent_part floating_suffix? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_fractional_constant() {
    // MORPHEME
    // hexadecimal_fractional_constant <- hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence / hexadecimal_digit_sequence '.'
    var $$;
    _beginToken(38);  
    // => hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence / hexadecimal_digit_sequence '.' # Choice
    switch (_getState(_transitions32)) {
      // [.]
      case 0:
        // => hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => hexadecimal_digit_sequence?
          var testing0 = _testing;
          _testing = _cursor;
          // => hexadecimal_digit_sequence
          $$ = _parse_hexadecimal_digit_sequence();
          // <= hexadecimal_digit_sequence
          success = true; 
          _testing = testing0;
          // <= hexadecimal_digit_sequence?
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => '.'
          $$ = _matchChar(46, '.');
          // <= '.'
          if (!success) break;
          seq[1] = $$;
          // => hexadecimal_digit_sequence
          $$ = _parse_hexadecimal_digit_sequence();
          // <= hexadecimal_digit_sequence
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence # Sequence
        break;
      // [0-a] [f]
      case 1:
        while (true) {
          // => hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_digit_sequence?
            var testing1 = _testing;
            _testing = _cursor;
            // => hexadecimal_digit_sequence
            $$ = _parse_hexadecimal_digit_sequence();
            // <= hexadecimal_digit_sequence
            success = true; 
            _testing = testing1;
            // <= hexadecimal_digit_sequence?
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => '.'
            $$ = _matchChar(46, '.');
            // <= '.'
            if (!success) break;
            seq[1] = $$;
            // => hexadecimal_digit_sequence
            $$ = _parse_hexadecimal_digit_sequence();
            // <= hexadecimal_digit_sequence
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence # Sequence
          if (success) break;
          // => hexadecimal_digit_sequence '.' # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_digit_sequence
            $$ = _parse_hexadecimal_digit_sequence();
            // <= hexadecimal_digit_sequence
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => '.'
            $$ = _matchChar(46, '.');
            // <= '.'
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= hexadecimal_digit_sequence '.' # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_fractional_constant
      _failure(_expect78);
    }
    // <= hexadecimal_digit_sequence? '.' hexadecimal_digit_sequence / hexadecimal_digit_sequence '.' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_hexadecimal_prefix() {
    // MORPHEME
    // hexadecimal_prefix <- '0x' / '0X'
    var $$;
    _beginToken(39);  
    // => '0x' / '0X' # Choice
    switch (_ch == 48 ? 0 : _ch == -1 ? 2 : 1) {
      // [0]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => '0x'
          $$ = _matchString(_strings17, '0x');
          // <= '0x'
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => '0X'
          $$ = _matchString(_strings18, '0X');
          // <= '0X'
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: hexadecimal_prefix
      _failure(_expect79);
    }
    // <= '0x' / '0X' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_identifier() {
    // LEXEME & MORPHEME
    // identifier <- identifier_base spaces
    var $$;
    _beginToken(0);  
    // => identifier_base spaces # Choice
    switch (_getState(_transitions14)) {
      // [A-Z] [\\] [_] [a-z]
      case 0:
        // => identifier_base spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => identifier_base
          $$ = _parse_identifier_base();
          // <= identifier_base
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // identifier_base
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= identifier_base spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: identifier
      _failure(_expect26);
    }
    // <= identifier_base spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_identifier_base() {
    // MORPHEME
    // identifier_base <- identifier_nondigit identifier_base1
    var $$;
    _beginToken(40);  
    // => identifier_nondigit identifier_base1 # Choice
    switch (_getState(_transitions14)) {
      // [A-Z] [\\] [_] [a-z]
      case 0:
        // => identifier_nondigit identifier_base1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => identifier_nondigit
          $$ = _parse_identifier_nondigit();
          // <= identifier_nondigit
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => identifier_base1
          $$ = _parse_identifier_base1();
          // <= identifier_base1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // identifier_nondigit
            final $1 = seq[0];
            // identifier_base1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = new Identifier(name: _text(), position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= identifier_nondigit identifier_base1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: identifier_base
      _failure(_expect80);
    }
    // <= identifier_nondigit identifier_base1 # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_identifier_base1() {
    // MORPHEME
    // identifier_base1 <- identifier_nondigit identifier_base1 / digit identifier_base1 / ''
    var $$;
    _beginToken(41);  
    // => identifier_nondigit identifier_base1 / digit identifier_base1 / '' # Choice
    switch (_getState(_transitions33)) {
      // [\u0000-/] [:-@] [[] []-^] [`] [{-\u0010ffff]
      // EOF
      case 0:
      case 4:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        _startPos = startPos0;
        break;
      // [0-9]
      case 1:
        while (true) {
          // => digit identifier_base1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => digit
            $$ = _parse_digit();
            // <= digit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => identifier_base1
            $$ = _parse_identifier_base1();
            // <= identifier_base1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // digit
              final $1 = seq[0];
              // identifier_base1
              final $2 = seq[1];
              final $start = startPos1;
              $$ = _text();
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= digit identifier_base1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos2;
          break;
        }
        break;
      // [A-Z] [\\] [_] [a-z]
      case 2:
        while (true) {
          // => identifier_nondigit identifier_base1 # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos3 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => identifier_nondigit
            $$ = _parse_identifier_nondigit();
            // <= identifier_nondigit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => identifier_base1
            $$ = _parse_identifier_base1();
            // <= identifier_base1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // identifier_nondigit
              final $1 = seq[0];
              // identifier_base1
              final $2 = seq[1];
              final $start = startPos3;
              $$ = _text();
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos3;
          // <= identifier_nondigit identifier_base1 # Sequence
          if (success) break;
          var startPos4 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos4;
          break;
        }
        break;
      // No matches
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: identifier_base1
      _failure(_expect81);
    }
    // <= identifier_nondigit identifier_base1 / digit identifier_base1 / '' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_identifier_nondigit() {
    // MORPHEME
    // identifier_nondigit <- nondigit / universal_character_name
    var $$;
    _beginToken(42);  
    // => nondigit / universal_character_name # Choice
    switch (_getState(_transitions34)) {
      // [A-Z] [_] [a-z]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => nondigit
        $$ = _parse_nondigit();
        // <= nondigit
        _startPos = startPos0;
        break;
      // [\\]
      case 1:
        var startPos1 = _startPos;
        _startPos = _cursor;
        // => universal_character_name
        $$ = _parse_universal_character_name();
        // <= universal_character_name
        _startPos = startPos1;
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: identifier_nondigit
      _failure(_expect82);
    }
    // <= nondigit / universal_character_name # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_if() {
    // LEXEME
    // if <- '#if' spaces
    var $$;
    // => '#if' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#if' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#if'
          $$ = _matchString(_strings5, '#if');
          // <= '#if'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#if'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#if' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#if'
      _failure(_expect27);
    }
    // <= '#if' spaces # Choice
    return $$;
  }
  
  dynamic _parse_if_group() {
    // NONTERMINAL
    // if_group <- if constant_expression new_line group? / ifdef identifier new_line group? / ifndef identifier new_line group?
    var $$;
    // => if constant_expression new_line group? / ifdef identifier new_line group? / ifndef identifier new_line group? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        while (true) {
          // => if constant_expression new_line group? # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => if
            $$ = _parse_if();
            // <= if
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => constant_expression
            $$ = _parse_constant_expression();
            // <= constant_expression
            if (!success) break;
            seq[1] = $$;
            // => new_line
            $$ = _parse_new_line();
            // <= new_line
            if (!success) break;
            seq[2] = $$;
            // => group?
            var testing0 = _testing;
            _testing = _cursor;
            // => group
            $$ = _parse_group();
            // <= group
            success = true; 
            _testing = testing0;
            // <= group?
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // if
              final $1 = seq[0];
              // constant_expression
              final $2 = seq[1];
              // new_line
              final $3 = seq[2];
              // group?
              final $4 = seq[3];
              final $start = startPos0;
              $$ = new IfDirective(body: $4, condition: $2, name: $1, position: $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= if constant_expression new_line group? # Sequence
          if (success) break;
          // => ifdef identifier new_line group? # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ifdef
            $$ = _parse_ifdef();
            // <= ifdef
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => identifier
            $$ = _parse_identifier();
            // <= identifier
            if (!success) break;
            seq[1] = $$;
            // => new_line
            $$ = _parse_new_line();
            // <= new_line
            if (!success) break;
            seq[2] = $$;
            // => group?
            var testing1 = _testing;
            _testing = _cursor;
            // => group
            $$ = _parse_group();
            // <= group
            success = true; 
            _testing = testing1;
            // <= group?
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // ifdef
              final $1 = seq[0];
              // identifier
              final $2 = seq[1];
              // new_line
              final $3 = seq[2];
              // group?
              final $4 = seq[3];
              final $start = startPos1;
              $$ = new IfDirective(body: $4, condition: $2, name: $1, position: $start);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= ifdef identifier new_line group? # Sequence
          if (success) break;
          // => ifndef identifier new_line group? # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ifndef
            $$ = _parse_ifndef();
            // <= ifndef
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => identifier
            $$ = _parse_identifier();
            // <= identifier
            if (!success) break;
            seq[1] = $$;
            // => new_line
            $$ = _parse_new_line();
            // <= new_line
            if (!success) break;
            seq[2] = $$;
            // => group?
            var testing2 = _testing;
            _testing = _cursor;
            // => group
            $$ = _parse_group();
            // <= group
            success = true; 
            _testing = testing2;
            // <= group?
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // ifndef
              final $1 = seq[0];
              // identifier
              final $2 = seq[1];
              // new_line
              final $3 = seq[2];
              // group?
              final $4 = seq[3];
              final $start = startPos2;
              $$ = new IfDirective(body: $4, condition: $2, name: $1, position: $start);
            }
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= ifndef identifier new_line group? # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#if', '#ifdef', '#ifndef'
      _failure(_expect2);
    }
    // <= if constant_expression new_line group? / ifdef identifier new_line group? / ifndef identifier new_line group? # Choice
    return $$;
  }
  
  dynamic _parse_if_section() {
    // NONTERMINAL
    // if_section <- if_group elif_groups? else_group? endif_line
    var $$;
    // => if_group elif_groups? else_group? endif_line # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => if_group elif_groups? else_group? endif_line # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => if_group
          $$ = _parse_if_group();
          // <= if_group
          if (!success) break;
          var seq = new List(4)..[0] = $$;
          // => elif_groups?
          var testing0 = _testing;
          _testing = _cursor;
          // => elif_groups
          $$ = _parse_elif_groups();
          // <= elif_groups
          success = true; 
          _testing = testing0;
          // <= elif_groups?
          if (!success) break;
          seq[1] = $$;
          // => else_group?
          var testing1 = _testing;
          _testing = _cursor;
          // => else_group
          $$ = _parse_else_group();
          // <= else_group
          success = true; 
          _testing = testing1;
          // <= else_group?
          if (!success) break;
          seq[2] = $$;
          // => endif_line
          $$ = _parse_endif_line();
          // <= endif_line
          if (!success) break;
          seq[3] = $$;
          $$ = seq;
          if (success) {    
            // if_group
            final $1 = seq[0];
            // elif_groups?
            final $2 = seq[1];
            // else_group?
            final $3 = seq[2];
            // endif_line
            final $4 = seq[3];
            final $start = startPos0;
            $$ = new IfSection(ifGroup: $1, elifGroups: $2, elseGroup: $3, endifLine: $4, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= if_group elif_groups? else_group? endif_line # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#if', '#ifdef', '#ifndef'
      _failure(_expect2);
    }
    // <= if_group elif_groups? else_group? endif_line # Choice
    return $$;
  }
  
  dynamic _parse_ifdef() {
    // LEXEME
    // ifdef <- '#ifdef' spaces
    var $$;
    // => '#ifdef' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#ifdef' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#ifdef'
          $$ = _matchString(_strings6, '#ifdef');
          // <= '#ifdef'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#ifdef'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#ifdef' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#ifdef'
      _failure(_expect28);
    }
    // <= '#ifdef' spaces # Choice
    return $$;
  }
  
  dynamic _parse_ifndef() {
    // LEXEME
    // ifndef <- '#ifndef' spaces
    var $$;
    // => '#ifndef' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#ifndef' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#ifndef'
          $$ = _matchString(_strings7, '#ifndef');
          // <= '#ifndef'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#ifndef'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#ifndef' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#ifndef'
      _failure(_expect29);
    }
    // <= '#ifndef' spaces # Choice
    return $$;
  }
  
  dynamic _parse_inclusive_or_expression() {
    // NONTERMINAL
    // inclusive_or_expression <- exclusive_or_expression inclusive_or_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[9] >= pos) {
      $$ = _getFromCache(9);
    } else {
      _cachePos[9] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => exclusive_or_expression inclusive_or_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => exclusive_or_expression inclusive_or_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => exclusive_or_expression
          $$ = _parse_exclusive_or_expression();
          // <= exclusive_or_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => inclusive_or_expression1
          $$ = _parse_inclusive_or_expression1();
          // <= inclusive_or_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // exclusive_or_expression
            final $1 = seq[0];
            // inclusive_or_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= exclusive_or_expression inclusive_or_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= exclusive_or_expression inclusive_or_expression1 # Choice
    if (_cacheable[9]) {
      _addToCache($$, pos, 9);
    }  
    return $$;
  }
  
  dynamic _parse_inclusive_or_expression1() {
    // NONTERMINAL
    // inclusive_or_expression1 <- vertical_line exclusive_or_expression inclusive_or_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[27] >= pos) {
      $$ = _getFromCache(27);
    } else {
      _cachePos[27] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => vertical_line exclusive_or_expression inclusive_or_expression1 / '' # Choice
    switch (_getState(_transitions11)) {
      // [\u0000-{] [}-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [|]
      case 1:
        while (true) {
          // => vertical_line exclusive_or_expression inclusive_or_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => vertical_line
            $$ = _parse_vertical_line();
            // <= vertical_line
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => exclusive_or_expression
            $$ = _parse_exclusive_or_expression();
            // <= exclusive_or_expression
            if (!success) break;
            seq[1] = $$;
            // => inclusive_or_expression1
            $$ = _parse_inclusive_or_expression1();
            // <= inclusive_or_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // vertical_line
              final $1 = seq[0];
              // exclusive_or_expression
              final $2 = seq[1];
              // inclusive_or_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= vertical_line exclusive_or_expression inclusive_or_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '|', null
      _failure(_expect12);
    }
    // <= vertical_line exclusive_or_expression inclusive_or_expression1 / '' # Choice
    if (_cacheable[27]) {
      _addToCache($$, pos, 27);
    }  
    return $$;
  }
  
  dynamic _parse_integer_constant_base() {
    // MORPHEME
    // integer_constant_base <- decimal_constant integer_suffix? / hexadecimal_constant integer_suffix? / octal_constant integer_suffix?
    var $$;
    _beginToken(43);  
    // => decimal_constant integer_suffix? / hexadecimal_constant integer_suffix? / octal_constant integer_suffix? # Choice
    switch (_getState(_transitions35)) {
      // [0]
      case 0:
        while (true) {
          // => hexadecimal_constant integer_suffix? # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => hexadecimal_constant
            $$ = _parse_hexadecimal_constant();
            // <= hexadecimal_constant
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => integer_suffix?
            var testing0 = _testing;
            _testing = _cursor;
            // => integer_suffix
            $$ = _parse_integer_suffix();
            // <= integer_suffix
            success = true; 
            _testing = testing0;
            // <= integer_suffix?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // hexadecimal_constant
              final $1 = seq[0];
              // integer_suffix?
              final $2 = seq[1];
              final $start = startPos0;
              $$ = new IntegerLiteral(position: $start, text: _text(), value: $1);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= hexadecimal_constant integer_suffix? # Sequence
          if (success) break;
          // => octal_constant integer_suffix? # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => octal_constant
            $$ = _parse_octal_constant();
            // <= octal_constant
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => integer_suffix?
            var testing1 = _testing;
            _testing = _cursor;
            // => integer_suffix
            $$ = _parse_integer_suffix();
            // <= integer_suffix
            success = true; 
            _testing = testing1;
            // <= integer_suffix?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // octal_constant
              final $1 = seq[0];
              // integer_suffix?
              final $2 = seq[1];
              final $start = startPos1;
              $$ = new IntegerLiteral(position: $start, text: _text(), value: $1);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= octal_constant integer_suffix? # Sequence
          break;
        }
        break;
      // [1-9]
      case 1:
        // => decimal_constant integer_suffix? # Sequence
        var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => decimal_constant
          $$ = _parse_decimal_constant();
          // <= decimal_constant
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => integer_suffix?
          var testing2 = _testing;
          _testing = _cursor;
          // => integer_suffix
          $$ = _parse_integer_suffix();
          // <= integer_suffix
          success = true; 
          _testing = testing2;
          // <= integer_suffix?
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // decimal_constant
            final $1 = seq[0];
            // integer_suffix?
            final $2 = seq[1];
            final $start = startPos2;
            $$ = new IntegerLiteral(position: $start, text: _text(), value: $1);
          }
          break;
        }
        if (!success) {
          _ch = ch2;
          _cursor = pos2;
        }
        _startPos = startPos2;
        // <= decimal_constant integer_suffix? # Sequence
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: integer_constant_base
      _failure(_expect83);
    }
    // <= decimal_constant integer_suffix? / hexadecimal_constant integer_suffix? / octal_constant integer_suffix? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_integer_suffix() {
    // MORPHEME
    // integer_suffix <- unsigned_suffix long_long_suffix / unsigned_suffix long_suffix? / long_long_suffix unsigned_suffix? / long_suffix unsigned_suffix?
    var $$;
    _beginToken(44);  
    // => unsigned_suffix long_long_suffix / unsigned_suffix long_suffix? / long_long_suffix unsigned_suffix? / long_suffix unsigned_suffix? # Choice
    switch (_getState(_transitions36)) {
      // [L] [l]
      case 0:
        while (true) {
          // => long_long_suffix unsigned_suffix? # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => long_long_suffix
            $$ = _parse_long_long_suffix();
            // <= long_long_suffix
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => unsigned_suffix?
            var testing0 = _testing;
            _testing = _cursor;
            // => unsigned_suffix
            $$ = _parse_unsigned_suffix();
            // <= unsigned_suffix
            success = true; 
            _testing = testing0;
            // <= unsigned_suffix?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= long_long_suffix unsigned_suffix? # Sequence
          if (success) break;
          // => long_suffix unsigned_suffix? # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => long_suffix
            $$ = _parse_long_suffix();
            // <= long_suffix
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => unsigned_suffix?
            var testing1 = _testing;
            _testing = _cursor;
            // => unsigned_suffix
            $$ = _parse_unsigned_suffix();
            // <= unsigned_suffix
            success = true; 
            _testing = testing1;
            // <= unsigned_suffix?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= long_suffix unsigned_suffix? # Sequence
          break;
        }
        break;
      // [U] [u]
      case 1:
        while (true) {
          // => unsigned_suffix long_long_suffix # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => unsigned_suffix
            $$ = _parse_unsigned_suffix();
            // <= unsigned_suffix
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => long_long_suffix
            $$ = _parse_long_long_suffix();
            // <= long_long_suffix
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= unsigned_suffix long_long_suffix # Sequence
          if (success) break;
          // => unsigned_suffix long_suffix? # Sequence
          var ch3 = _ch, pos3 = _cursor, startPos3 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => unsigned_suffix
            $$ = _parse_unsigned_suffix();
            // <= unsigned_suffix
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => long_suffix?
            var testing2 = _testing;
            _testing = _cursor;
            // => long_suffix
            $$ = _parse_long_suffix();
            // <= long_suffix
            success = true; 
            _testing = testing2;
            // <= long_suffix?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch3;
            _cursor = pos3;
          }
          _startPos = startPos3;
          // <= unsigned_suffix long_suffix? # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: integer_suffix
      _failure(_expect84);
    }
    // <= unsigned_suffix long_long_suffix / unsigned_suffix long_suffix? / long_long_suffix unsigned_suffix? / long_suffix unsigned_suffix? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_logical_and_expression() {
    // NONTERMINAL
    // logical_and_expression <- inclusive_or_expression logical_and_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[8] >= pos) {
      $$ = _getFromCache(8);
    } else {
      _cachePos[8] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => inclusive_or_expression logical_and_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => inclusive_or_expression logical_and_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => inclusive_or_expression
          $$ = _parse_inclusive_or_expression();
          // <= inclusive_or_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => logical_and_expression1
          $$ = _parse_logical_and_expression1();
          // <= logical_and_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // inclusive_or_expression
            final $1 = seq[0];
            // logical_and_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= inclusive_or_expression logical_and_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= inclusive_or_expression logical_and_expression1 # Choice
    if (_cacheable[8]) {
      _addToCache($$, pos, 8);
    }  
    return $$;
  }
  
  dynamic _parse_logical_and_expression1() {
    // NONTERMINAL
    // logical_and_expression1 <- ampersand2 inclusive_or_expression logical_or_expression1 / ''
    var $$;
    // => ampersand2 inclusive_or_expression logical_or_expression1 / '' # Choice
    switch (_getState(_transitions9)) {
      // [\u0000-%] [\'-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [&]
      case 1:
        while (true) {
          // => ampersand2 inclusive_or_expression logical_or_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ampersand2
            $$ = _parse_ampersand2();
            // <= ampersand2
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => inclusive_or_expression
            $$ = _parse_inclusive_or_expression();
            // <= inclusive_or_expression
            if (!success) break;
            seq[1] = $$;
            // => logical_or_expression1
            $$ = _parse_logical_or_expression1();
            // <= logical_or_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // ampersand2
              final $1 = seq[0];
              // inclusive_or_expression
              final $2 = seq[1];
              // logical_or_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= ampersand2 inclusive_or_expression logical_or_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '&&', null
      _failure(_expect13);
    }
    // <= ampersand2 inclusive_or_expression logical_or_expression1 / '' # Choice
    return $$;
  }
  
  dynamic _parse_logical_or_expression() {
    // NONTERMINAL
    // logical_or_expression <- logical_and_expression logical_or_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[7] >= pos) {
      $$ = _getFromCache(7);
    } else {
      _cachePos[7] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => logical_and_expression logical_or_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => logical_and_expression logical_or_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => logical_and_expression
          $$ = _parse_logical_and_expression();
          // <= logical_and_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => logical_or_expression1
          $$ = _parse_logical_or_expression1();
          // <= logical_or_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // logical_and_expression
            final $1 = seq[0];
            // logical_or_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= logical_and_expression logical_or_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= logical_and_expression logical_or_expression1 # Choice
    if (_cacheable[7]) {
      _addToCache($$, pos, 7);
    }  
    return $$;
  }
  
  dynamic _parse_logical_or_expression1() {
    // NONTERMINAL
    // logical_or_expression1 <- vertical_line2 logical_and_expression logical_or_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[29] >= pos) {
      $$ = _getFromCache(29);
    } else {
      _cachePos[29] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => vertical_line2 logical_and_expression logical_or_expression1 / '' # Choice
    switch (_getState(_transitions11)) {
      // [\u0000-{] [}-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [|]
      case 1:
        while (true) {
          // => vertical_line2 logical_and_expression logical_or_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => vertical_line2
            $$ = _parse_vertical_line2();
            // <= vertical_line2
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => logical_and_expression
            $$ = _parse_logical_and_expression();
            // <= logical_and_expression
            if (!success) break;
            seq[1] = $$;
            // => logical_or_expression1
            $$ = _parse_logical_or_expression1();
            // <= logical_or_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // vertical_line2
              final $1 = seq[0];
              // logical_and_expression
              final $2 = seq[1];
              // logical_or_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= vertical_line2 logical_and_expression logical_or_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '||', null
      _failure(_expect14);
    }
    // <= vertical_line2 logical_and_expression logical_or_expression1 / '' # Choice
    if (_cacheable[29]) {
      _addToCache($$, pos, 29);
    }  
    return $$;
  }
  
  dynamic _parse_long_long_suffix() {
    // MORPHEME
    // long_long_suffix <- 'll' / 'LL'
    var $$;
    _beginToken(45);  
    // => 'll' / 'LL' # Choice
    switch (_getState(_transitions37)) {
      // [L]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => 'LL'
        $$ = _matchString(_strings19, 'LL');
        // <= 'LL'
        _startPos = startPos0;
        break;
      // [l]
      case 1:
        var startPos1 = _startPos;
        _startPos = _cursor;
        // => 'll'
        $$ = _matchString(_strings20, 'll');
        // <= 'll'
        _startPos = startPos1;
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: long_long_suffix
      _failure(_expect85);
    }
    // <= 'll' / 'LL' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_long_suffix() {
    // MORPHEME
    // long_suffix <- [Ll]
    var $$;
    _beginToken(46);  
    // => [Ll] # Choice
    switch (_getState(_transitions38)) {
      // [L] [l]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [Ll]
        $$ = _matchMapping(76, 108, _mapping4);
        // <= [Ll]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: long_suffix
      _failure(_expect86);
    }
    // <= [Ll] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_lparen() {
    // LEXEME & MORPHEME
    // lparen <- '(' spaces
    var $$;
    _beginToken(1);  
    // => '(' spaces # Choice
    switch (_ch == 40 ? 0 : _ch == -1 ? 2 : 1) {
      // [(]
      case 0:
        // => '(' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '('
          $$ = '(';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '('
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '('
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '(' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '('
      _failure(_expect30);
    }
    // <= '(' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_lsh() {
    // MORPHEME
    // lsh <- '<<' spaces
    var $$;
    _beginToken(47);  
    // => '<<' spaces # Choice
    switch (_ch == 60 ? 0 : _ch == -1 ? 2 : 1) {
      // [<]
      case 0:
        // => '<<' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '<<'
          $$ = _matchString(_strings21, '<<');
          // <= '<<'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '<<'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '<<' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<<'
      _failure(_expect38);
    }
    // <= '<<' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_lt() {
    // MORPHEME
    // lt <- '<' spaces
    var $$;
    _beginToken(48);  
    // => '<' spaces # Choice
    switch (_ch == 60 ? 0 : _ch == -1 ? 2 : 1) {
      // [<]
      case 0:
        // => '<' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '<'
          $$ = '<';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '<'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '<'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '<' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<'
      _failure(_expect87);
    }
    // <= '<' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_lte() {
    // MORPHEME
    // lte <- '<=' spaces
    var $$;
    _beginToken(49);  
    // => '<=' spaces # Choice
    switch (_ch == 60 ? 0 : _ch == -1 ? 2 : 1) {
      // [<]
      case 0:
        // => '<=' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '<='
          $$ = _matchString(_strings22, '<=');
          // <= '<='
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '<='
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '<=' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<='
      _failure(_expect88);
    }
    // <= '<=' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_minus() {
    // MORPHEME
    // minus <- '+' spaces
    var $$;
    _beginToken(50);  
    // => '+' spaces # Choice
    switch (_ch == 43 ? 0 : _ch == -1 ? 2 : 1) {
      // [+]
      case 0:
        // => '+' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '+'
          $$ = '+';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '+'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '+'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '+' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '+'
      _failure(_expect18);
    }
    // <= '+' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_multiplicative_expression() {
    // NONTERMINAL
    // multiplicative_expression <- unary_expression multiplicative_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[16] >= pos) {
      $$ = _getFromCache(16);
    } else {
      _cachePos[16] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => unary_expression multiplicative_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => unary_expression multiplicative_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => unary_expression
          $$ = _parse_unary_expression();
          // <= unary_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => multiplicative_expression1
          $$ = _parse_multiplicative_expression1();
          // <= multiplicative_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // unary_expression
            final $1 = seq[0];
            // multiplicative_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= unary_expression multiplicative_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= unary_expression multiplicative_expression1 # Choice
    if (_cacheable[16]) {
      _addToCache($$, pos, 16);
    }  
    return $$;
  }
  
  dynamic _parse_multiplicative_expression1() {
    // NONTERMINAL
    // multiplicative_expression1 <- multiplicative_operator unary_expression multiplicative_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[20] >= pos) {
      $$ = _getFromCache(20);
    } else {
      _cachePos[20] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => multiplicative_operator unary_expression multiplicative_expression1 / '' # Choice
    switch (_getState(_transitions4)) {
      // [\u0000-\$] [&-)] [+-.] [0-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [%] [*] [/]
      case 1:
        while (true) {
          // => multiplicative_operator unary_expression multiplicative_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => multiplicative_operator
            $$ = _parse_multiplicative_operator();
            // <= multiplicative_operator
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => unary_expression
            $$ = _parse_unary_expression();
            // <= unary_expression
            if (!success) break;
            seq[1] = $$;
            // => multiplicative_expression1
            $$ = _parse_multiplicative_expression1();
            // <= multiplicative_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // multiplicative_operator
              final $1 = seq[0];
              // unary_expression
              final $2 = seq[1];
              // multiplicative_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= multiplicative_operator unary_expression multiplicative_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: multiplicative_operator, null
      _failure(_expect5);
    }
    // <= multiplicative_operator unary_expression multiplicative_expression1 / '' # Choice
    if (_cacheable[20]) {
      _addToCache($$, pos, 20);
    }  
    return $$;
  }
  
  dynamic _parse_multiplicative_operator() {
    // LEXEME
    // multiplicative_operator <- asterisk / slash / percent
    var $$;
    // => asterisk / slash / percent # Choice
    switch (_getState(_transitions15)) {
      // [%]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => percent
        $$ = _parse_percent();
        // <= percent
        _startPos = startPos0;
        break;
      // [*]
      case 1:
        var startPos1 = _startPos;
        _startPos = _cursor;
        // => asterisk
        $$ = _parse_asterisk();
        // <= asterisk
        _startPos = startPos1;
        break;
      // [/]
      case 2:
        var startPos2 = _startPos;
        _startPos = _cursor;
        // => slash
        $$ = _parse_slash();
        // <= slash
        _startPos = startPos2;
        break;
      // No matches
      // EOF
      case 3:
      case 4:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: multiplicative_operator
      _failure(_expect31);
    }
    // <= asterisk / slash / percent # Choice
    return $$;
  }
  
  dynamic _parse_neq() {
    // LEXEME
    // neq <- '!=' spaces
    var $$;
    // => '!=' spaces # Choice
    switch (_ch == 33 ? 0 : _ch == -1 ? 2 : 1) {
      // [!]
      case 0:
        // => '!=' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '!='
          $$ = _matchString(_strings8, '!=');
          // <= '!='
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '!='
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '!=' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '!='
      _failure(_expect32);
    }
    // <= '!=' spaces # Choice
    return $$;
  }
  
  dynamic _parse_new_line() {
    // LEXEME & MORPHEME
    // new_line <- '\r\n' / [\n\r]
    var $$;
    _beginToken(2);  
    // => '\r\n' / [\n\r] # Choice
    switch (_getState(_transitions16)) {
      // [\n]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [\n\r]
        $$ = _matchMapping(10, 13, _mapping0);
        // <= [\n\r]
        _startPos = startPos0;
        break;
      // [\r]
      case 1:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => '\r\n'
          $$ = _matchString(_strings9, '\r\n');
          // <= '\r\n'
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => [\n\r]
          $$ = _matchMapping(10, 13, _mapping0);
          // <= [\n\r]
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: new_line
      _failure(_expect33);
    }
    // <= '\r\n' / [\n\r] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_new_line_fragment() {
    // MORPHEME
    // new_line_fragment <- new_line
    var $$;
    _beginToken(51);  
    // => new_line # Choice
    switch (_getState(_transitions39)) {
      // [\n] [\r]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => new_line
        $$ = _parse_new_line();
        // <= new_line
        if (success) {    
          // new_line
          final $1 = $$;
          final $start = startPos0;
          $$ = new SourceFragment(position: $start, text: _text());
        }
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: new_line_fragment
      _failure(_expect89);
    }
    // <= new_line # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_nondigit() {
    // MORPHEME
    // nondigit <- [A-Z_a-z]
    var $$;
    _beginToken(52);  
    // => [A-Z_a-z] # Choice
    switch (_getState(_transitions40)) {
      // [A-Z] [_] [a-z]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [A-Z_a-z]
        $$ = _matchMapping(65, 122, _mapping5);
        // <= [A-Z_a-z]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: nondigit
      _failure(_expect90);
    }
    // <= [A-Z_a-z] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_nonzero_digit() {
    // MORPHEME
    // nonzero_digit <- [1-9]
    var $$;
    _beginToken(53);  
    // => [1-9] # Choice
    switch (_ch >= 49 && _ch <= 57 ? 0 : _ch == -1 ? 2 : 1) {
      // [1-9]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [1-9]
        $$ = _matchRange(49, 57);
        // <= [1-9]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: nonzero_digit
      _failure(_expect91);
    }
    // <= [1-9] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_octal_constant() {
    // MORPHEME
    // octal_constant <- '0' octal_constant1
    var $$;
    _beginToken(54);  
    // => '0' octal_constant1 # Choice
    switch (_ch == 48 ? 0 : _ch == -1 ? 2 : 1) {
      // [0]
      case 0:
        // => '0' octal_constant1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '0'
          $$ = '0';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '0'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => octal_constant1
          $$ = _parse_octal_constant1();
          // <= octal_constant1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '0'
            final $1 = seq[0];
            // octal_constant1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = int.parse(_text(), radix: 8);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '0' octal_constant1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '0'
      _failure(_expect92);
    }
    // <= '0' octal_constant1 # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_octal_constant1() {
    // MORPHEME
    // octal_constant1 <- octal_digit octal_constant1 / ''
    var $$;
    _beginToken(55);  
    // => octal_digit octal_constant1 / '' # Choice
    switch (_getState(_transitions41)) {
      // [\u0000-/] [8-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        _startPos = startPos0;
        break;
      // [0-7]
      case 1:
        while (true) {
          // => octal_digit octal_constant1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => octal_constant1
            $$ = _parse_octal_constant1();
            // <= octal_constant1
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= octal_digit octal_constant1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: octal_constant1
      _failure(_expect93);
    }
    // <= octal_digit octal_constant1 / '' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_octal_digit() {
    // MORPHEME
    // octal_digit <- [0-7]
    var $$;
    _beginToken(56);  
    // => [0-7] # Choice
    switch (_ch >= 48 && _ch <= 55 ? 0 : _ch == -1 ? 2 : 1) {
      // [0-7]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [0-7]
        $$ = _matchRange(48, 55);
        // <= [0-7]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: octal_digit
      _failure(_expect94);
    }
    // <= [0-7] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_octal_escape_sequence() {
    // MORPHEME
    // octal_escape_sequence <- '\\' octal_digit octal_digit octal_digit / '\\' octal_digit octal_digit / '\\' octal_digit
    var $$;
    _beginToken(57);  
    // => '\\' octal_digit octal_digit octal_digit / '\\' octal_digit octal_digit / '\\' octal_digit # Choice
    switch (_ch == 92 ? 0 : _ch == -1 ? 2 : 1) {
      // [\\]
      case 0:
        while (true) {
          // => '\\' octal_digit octal_digit octal_digit # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => '\\'
            $$ = '\\';
            success = true;
            if (++_cursor < _inputLen) {
              _ch = _input[_cursor];
            } else {
              _ch = -1;
            }
            // <= '\\'
            if (!success) break;
            var seq = new List(4)..[0] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[1] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[2] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[3] = $$;
            $$ = seq;
            if (success) {    
              // '\\'
              final $1 = seq[0];
              // octal_digit
              final $2 = seq[1];
              // octal_digit
              final $3 = seq[2];
              // octal_digit
              final $4 = seq[3];
              final $start = startPos0;
              $$ = int.parse(_text(1), radix: 8);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= '\\' octal_digit octal_digit octal_digit # Sequence
          if (success) break;
          // => '\\' octal_digit octal_digit # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => '\\'
            $$ = '\\';
            success = true;
            if (++_cursor < _inputLen) {
              _ch = _input[_cursor];
            } else {
              _ch = -1;
            }
            // <= '\\'
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[1] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // '\\'
              final $1 = seq[0];
              // octal_digit
              final $2 = seq[1];
              // octal_digit
              final $3 = seq[2];
              final $start = startPos1;
              $$ = int.parse(_text(1), radix: 8);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= '\\' octal_digit octal_digit # Sequence
          if (success) break;
          // => '\\' octal_digit # Sequence
          var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => '\\'
            $$ = '\\';
            success = true;
            if (++_cursor < _inputLen) {
              _ch = _input[_cursor];
            } else {
              _ch = -1;
            }
            // <= '\\'
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => octal_digit
            $$ = _parse_octal_digit();
            // <= octal_digit
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // '\\'
              final $1 = seq[0];
              // octal_digit
              final $2 = seq[1];
              final $start = startPos2;
              $$ = int.parse($2, radix: 8);
            }
            break;
          }
          if (!success) {
            _ch = ch2;
            _cursor = pos2;
          }
          _startPos = startPos2;
          // <= '\\' octal_digit # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '\'
      _failure(_expect95);
    }
    // <= '\\' octal_digit octal_digit octal_digit / '\\' octal_digit octal_digit / '\\' octal_digit # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_percent() {
    // MORPHEME
    // percent <- '%' spaces
    var $$;
    _beginToken(58);  
    // => '%' spaces # Choice
    switch (_ch == 37 ? 0 : _ch == -1 ? 2 : 1) {
      // [%]
      case 0:
        // => '%' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '%'
          $$ = '%';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '%'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '%'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '%' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '%'
      _failure(_expect96);
    }
    // <= '%' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_plus() {
    // MORPHEME
    // plus <- '+' spaces
    var $$;
    _beginToken(59);  
    // => '+' spaces # Choice
    switch (_ch == 43 ? 0 : _ch == -1 ? 2 : 1) {
      // [+]
      case 0:
        // => '+' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '+'
          $$ = '+';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '+'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '+'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '+' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '+'
      _failure(_expect18);
    }
    // <= '+' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_primary_expression() {
    // NONTERMINAL
    // primary_expression <- defined / identifier / constant / lparen expression rparen
    var $$;
    // => defined / identifier / constant / lparen expression rparen # Choice
    switch (_getState(_transitions3)) {
      // [\'] [.] [0-9]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => constant
        $$ = _parse_constant();
        // <= constant
        _startPos = startPos0;
        break;
      // [(]
      case 1:
        // => lparen expression rparen # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => lparen
          $$ = _parse_lparen();
          // <= lparen
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => expression
          $$ = _parse_expression();
          // <= expression
          if (!success) break;
          seq[1] = $$;
          // => rparen
          $$ = _parse_rparen();
          // <= rparen
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // lparen
            final $1 = seq[0];
            // expression
            final $2 = seq[1];
            // rparen
            final $3 = seq[2];
            final $start = startPos1;
            $$ = new ParenthesisExpression(expression: $2, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos1;
        // <= lparen expression rparen # Sequence
        break;
      // [A-Z]
      case 2:
        while (true) {
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => identifier
          $$ = _parse_identifier();
          // <= identifier
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => constant
          $$ = _parse_constant();
          // <= constant
          _startPos = startPos3;
          break;
        }
        break;
      // [\\] [_] [a-c] [e-z]
      case 3:
        var startPos4 = _startPos;
        _startPos = _cursor;
        // => identifier
        $$ = _parse_identifier();
        // <= identifier
        _startPos = startPos4;
        break;
      // [d]
      case 4:
        while (true) {
          var startPos5 = _startPos;
          _startPos = _cursor;
          // => defined
          $$ = _parse_defined();
          // <= defined
          _startPos = startPos5;
          if (success) break;
          var startPos6 = _startPos;
          _startPos = _cursor;
          // => identifier
          $$ = _parse_identifier();
          // <= identifier
          _startPos = startPos6;
          break;
        }
        break;
      // No matches
      // EOF
      case 5:
      case 6:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '('
      _failure(_expect4);
    }
    // <= defined / identifier / constant / lparen expression rparen # Choice
    return $$;
  }
  
  dynamic _parse_question_mark() {
    // LEXEME
    // question_mark <- '?' spaces
    var $$;
    // => '?' spaces # Choice
    switch (_ch == 63 ? 0 : _ch == -1 ? 2 : 1) {
      // [?]
      case 0:
        // => '?' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '?'
          $$ = '?';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '?'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '?'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '?' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '?'
      _failure(_expect34);
    }
    // <= '?' spaces # Choice
    return $$;
  }
  
  dynamic _parse_relational_expression() {
    // NONTERMINAL
    // relational_expression <- shift_expression relational_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[13] >= pos) {
      $$ = _getFromCache(13);
    } else {
      _cachePos[13] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => shift_expression relational_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => shift_expression relational_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => shift_expression
          $$ = _parse_shift_expression();
          // <= shift_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => relational_expression1
          $$ = _parse_relational_expression1();
          // <= relational_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // shift_expression
            final $1 = seq[0];
            // relational_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= shift_expression relational_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= shift_expression relational_expression1 # Choice
    if (_cacheable[13]) {
      _addToCache($$, pos, 13);
    }  
    return $$;
  }
  
  dynamic _parse_relational_expression1() {
    // NONTERMINAL
    // relational_expression1 <- relational_operator shift_expression relational_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[23] >= pos) {
      $$ = _getFromCache(23);
    } else {
      _cachePos[23] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => relational_operator shift_expression relational_expression1 / '' # Choice
    switch (_getState(_transitions7)) {
      // [\u0000-;] [=] [?-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [<] [>]
      case 1:
        while (true) {
          // => relational_operator shift_expression relational_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => relational_operator
            $$ = _parse_relational_operator();
            // <= relational_operator
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => shift_expression
            $$ = _parse_shift_expression();
            // <= shift_expression
            if (!success) break;
            seq[1] = $$;
            // => relational_expression1
            $$ = _parse_relational_expression1();
            // <= relational_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // relational_operator
              final $1 = seq[0];
              // shift_expression
              final $2 = seq[1];
              // relational_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= relational_operator shift_expression relational_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: relational_operator, null
      _failure(_expect8);
    }
    // <= relational_operator shift_expression relational_expression1 / '' # Choice
    if (_cacheable[23]) {
      _addToCache($$, pos, 23);
    }  
    return $$;
  }
  
  dynamic _parse_relational_operator() {
    // LEXEME
    // relational_operator <- lt / gt / lte / gte
    var $$;
    // => lt / gt / lte / gte # Choice
    switch (_getState(_transitions17)) {
      // [<]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => lt
          $$ = _parse_lt();
          // <= lt
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => lte
          $$ = _parse_lte();
          // <= lte
          _startPos = startPos1;
          break;
        }
        break;
      // [>]
      case 1:
        while (true) {
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => gt
          $$ = _parse_gt();
          // <= gt
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => gte
          $$ = _parse_gte();
          // <= gte
          _startPos = startPos3;
          break;
        }
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: relational_operator
      _failure(_expect35);
    }
    // <= lt / gt / lte / gte # Choice
    return $$;
  }
  
  dynamic _parse_replacement_list() {
    // MORPHEME
    // replacement_list <- source_fragment*
    var $$;
    _beginToken(60);  
    // => source_fragment* # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => source_fragment*
        var testing0 = _testing; 
        for (var reps = []; ; ) {
          _testing = _cursor;
          // => source_fragment
          $$ = _parse_source_fragment();
          // <= source_fragment
          if (success) {  
            reps.add($$);
          } else {
            success = true;
            _testing = testing0;
            $$ = reps;
            break; 
          }
        }
        // <= source_fragment*
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = true;
        break;
      // EOF
      case 2:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => source_fragment*
          var testing1 = _testing; 
          for (var reps = []; ; ) {
            _testing = _cursor;
            // => source_fragment
            $$ = _parse_source_fragment();
            // <= source_fragment
            if (success) {  
              reps.add($$);
            } else {
              success = true;
              _testing = testing1;
              $$ = reps;
              break; 
            }
          }
          // <= source_fragment*
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => source_fragment*
          var testing2 = _testing; 
          for (var reps = []; ; ) {
            _testing = _cursor;
            // => source_fragment
            $$ = _parse_source_fragment();
            // <= source_fragment
            if (success) {  
              reps.add($$);
            } else {
              success = true;
              _testing = testing2;
              $$ = reps;
              break; 
            }
          }
          // <= source_fragment*
          _startPos = startPos2;
          break;
        }
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: replacement_list
      _failure(_expect97);
    }
    // <= source_fragment* # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_rparen() {
    // LEXEME & MORPHEME
    // rparen <- ')' spaces
    var $$;
    _beginToken(3);  
    // => ')' spaces # Choice
    switch (_ch == 41 ? 0 : _ch == -1 ? 2 : 1) {
      // [)]
      case 0:
        // => ')' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ')'
          $$ = ')';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= ')'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // ')'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= ')' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: ')'
      _failure(_expect36);
    }
    // <= ')' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_rsh() {
    // MORPHEME
    // rsh <- '<<' spaces
    var $$;
    _beginToken(61);  
    // => '<<' spaces # Choice
    switch (_ch == 60 ? 0 : _ch == -1 ? 2 : 1) {
      // [<]
      case 0:
        // => '<<' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '<<'
          $$ = _matchString(_strings21, '<<');
          // <= '<<'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '<<'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '<<' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<<'
      _failure(_expect38);
    }
    // <= '<<' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_s_char() {
    // MORPHEME
    // s_char <- ![\n\r"\\] . / escape_sequence
    var $$;
    _beginToken(62);  
    // => ![\n\r"\\] . / escape_sequence # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      case 0:
        while (true) {
          // => ![\n\r"\\] . # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => ![\n\r"\\]
            var ch1 = _ch, pos1 = _cursor, testing0 = _testing; 
            _testing = _inputLen + 1;
            // => [\n\r"\\]
            $$ = _matchMapping(10, 92, _mapping6);
            // <= [\n\r"\\]
            _ch = ch1;
            _cursor = pos1; 
            _testing = testing0;
            $$ = null;
            success = !success;
            // <= ![\n\r"\\]
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => .
            $$ = _matchAny();
            // <= .
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // ![\n\r"\\]
              final $1 = seq[0];
              // .
              final $2 = seq[1];
              final $start = startPos0;
              $$ = $2.codeUnitAt(0);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= ![\n\r"\\] . # Sequence
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => escape_sequence
          $$ = _parse_escape_sequence();
          // <= escape_sequence
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
      // EOF
      case 2:
        // => ![\n\r"\\] . # Sequence
        var ch2 = _ch, pos2 = _cursor, startPos2 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ![\n\r"\\]
          var ch3 = _ch, pos3 = _cursor, testing1 = _testing; 
          _testing = _inputLen + 1;
          // => [\n\r"\\]
          $$ = _matchMapping(10, 92, _mapping6);
          // <= [\n\r"\\]
          _ch = ch3;
          _cursor = pos3; 
          _testing = testing1;
          $$ = null;
          success = !success;
          // <= ![\n\r"\\]
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => .
          $$ = _matchAny();
          // <= .
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // ![\n\r"\\]
            final $1 = seq[0];
            // .
            final $2 = seq[1];
            final $start = startPos2;
            $$ = $2.codeUnitAt(0);
          }
          break;
        }
        if (!success) {
          _ch = ch2;
          _cursor = pos2;
        }
        _startPos = startPos2;
        // <= ![\n\r"\\] . # Sequence
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: s_char
      _failure(_expect98);
    }
    // <= ![\n\r"\\] . / escape_sequence # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_s_char_sequence() {
    // MORPHEME
    // s_char_sequence <- s_char+
    var $$;
    _beginToken(63);  
    // => s_char+ # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => s_char+
        var testing0;
        for (var first = true, reps; ;) {  
          // => s_char  
          $$ = _parse_s_char();  
          // <= s_char  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= s_char+
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: s_char_sequence
      _failure(_expect99);
    }
    // <= s_char+ # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_semicolon() {
    // LEXEME
    // semicolon <- ':' spaces
    var $$;
    // => ':' spaces # Choice
    switch (_ch == 58 ? 0 : _ch == -1 ? 2 : 1) {
      // [:]
      case 0:
        // => ':' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => ':'
          $$ = ':';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= ':'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // ':'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= ':' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: ':'
      _failure(_expect37);
    }
    // <= ':' spaces # Choice
    return $$;
  }
  
  dynamic _parse_shift_expression() {
    // NONTERMINAL
    // shift_expression <- additive_expression shift_expression1
    var $$;          
    var pos = _cursor;    
    if(_cachePos[14] >= pos) {
      $$ = _getFromCache(14);
    } else {
      _cachePos[14] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => additive_expression shift_expression1 # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => additive_expression shift_expression1 # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => additive_expression
          $$ = _parse_additive_expression();
          // <= additive_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => shift_expression1
          $$ = _parse_shift_expression1();
          // <= shift_expression1
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // additive_expression
            final $1 = seq[0];
            // shift_expression1
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _binary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= additive_expression shift_expression1 # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= additive_expression shift_expression1 # Choice
    if (_cacheable[14]) {
      _addToCache($$, pos, 14);
    }  
    return $$;
  }
  
  dynamic _parse_shift_expression1() {
    // NONTERMINAL
    // shift_expression1 <- shift_operator additive_expression shift_expression1 / ''
    var $$;          
    var pos = _cursor;    
    if(_cachePos[22] >= pos) {
      $$ = _getFromCache(22);
    } else {
      _cachePos[22] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => shift_operator additive_expression shift_expression1 / '' # Choice
    switch (_getState(_transitions6)) {
      // [\u0000-;] [=-\u0010ffff]
      // EOF
      case 0:
      case 3:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => ''
        success = true;
        $$ = '';
        // <= ''
        if (success) {    
          // ''
          final $1 = $$;
          final $start = startPos0;
          $$ = null;
        }
        _startPos = startPos0;
        break;
      // [<]
      case 1:
        while (true) {
          // => shift_operator additive_expression shift_expression1 # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => shift_operator
            $$ = _parse_shift_operator();
            // <= shift_operator
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => additive_expression
            $$ = _parse_additive_expression();
            // <= additive_expression
            if (!success) break;
            seq[1] = $$;
            // => shift_expression1
            $$ = _parse_shift_expression1();
            // <= shift_expression1
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // shift_operator
              final $1 = seq[0];
              // additive_expression
              final $2 = seq[1];
              // shift_expression1
              final $3 = seq[2];
              final $start = startPos1;
              $$ = new _Node($1, $2, $3, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos1;
          // <= shift_operator additive_expression shift_expression1 # Sequence
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => ''
          success = true;
          $$ = '';
          // <= ''
          if (success) {    
            // ''
            final $1 = $$;
            final $start = startPos2;
            $$ = null;
          }
          _startPos = startPos2;
          break;
        }
        break;
      // No matches
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<<', null
      _failure(_expect7);
    }
    // <= shift_operator additive_expression shift_expression1 / '' # Choice
    if (_cacheable[22]) {
      _addToCache($$, pos, 22);
    }  
    return $$;
  }
  
  dynamic _parse_shift_operator() {
    // LEXEME
    // shift_operator <- lsh / rsh
    var $$;
    // => lsh / rsh # Choice
    switch (_ch == 60 ? 0 : _ch == -1 ? 2 : 1) {
      // [<]
      case 0:
        while (true) {
          var startPos0 = _startPos;
          _startPos = _cursor;
          // => lsh
          $$ = _parse_lsh();
          // <= lsh
          _startPos = startPos0;
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => rsh
          $$ = _parse_rsh();
          // <= rsh
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '<<'
      _failure(_expect38);
    }
    // <= lsh / rsh # Choice
    return $$;
  }
  
  dynamic _parse_sign() {
    // MORPHEME
    // sign <- [+\-]
    var $$;
    _beginToken(64);  
    // => [+\-] # Choice
    switch (_getState(_transitions42)) {
      // [+] [-]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [+\-]
        $$ = _matchMapping(43, 45, _mapping7);
        // <= [+\-]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: sign
      _failure(_expect100);
    }
    // <= [+\-] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_simple_escape_sequence() {
    // MORPHEME
    // simple_escape_sequence <- [\\] ["'?\\a-bfnrtv]
    var $$;
    _beginToken(65);  
    // => [\\] ["'?\\a-bfnrtv] # Choice
    switch (_ch == 92 ? 0 : _ch == -1 ? 2 : 1) {
      // [\\]
      case 0:
        // => [\\] ["'?\\a-bfnrtv] # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => [\\]
          $$ = '\\';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= [\\]
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => ["'?\\a-bfnrtv]
          $$ = _matchMapping(34, 118, _mapping8);
          // <= ["'?\\a-bfnrtv]
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // [\\]
            final $1 = seq[0];
            // ["'?\\a-bfnrtv]
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _escape($1);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= [\\] ["'?\\a-bfnrtv] # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: simple_escape_sequence
      _failure(_expect101);
    }
    // <= [\\] ["'?\\a-bfnrtv] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_slash() {
    // MORPHEME
    // slash <- '/' spaces
    var $$;
    _beginToken(66);  
    // => '/' spaces # Choice
    switch (_ch == 47 ? 0 : _ch == -1 ? 2 : 1) {
      // [/]
      case 0:
        // => '/' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '/'
          $$ = '/';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '/'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '/'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '/' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '/'
      _failure(_expect102);
    }
    // <= '/' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_source_fragment() {
    // MORPHEME
    // source_fragment <- string_literal_base / constant_base / identifier_base / (!new_line !identifier_base !string_literal_base !constant_base .)+
    var $$;
    _beginToken(67);  
    // => string_literal_base / constant_base / identifier_base / (!new_line !identifier_base !string_literal_base !constant_base .)+ # Choice
    switch (_getState(_transitions43)) {
      // [\u0000-!] [#-&] [(--] [/] [:-@] [[] []-^] [`] [{-\u0010ffff]
      // EOF
      case 0:
      case 6:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => (!new_line !identifier_base !string_literal_base !constant_base .)+
        var testing0;
        for (var first = true, reps; ;) {  
          // => (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
          switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {  
            // [\u0000-\u0010ffff]  
            // EOF  
            case 0:  
            case 2:  
              // => !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
              var ch0 = _ch, pos0 = _cursor, startPos1 = _startPos;  
              _startPos = _cursor;  
              while (true) {    
                // => !new_line  
                var ch1 = _ch, pos1 = _cursor, testing1 = _testing;   
                _testing = _inputLen + 1;  
                // => new_line  
                $$ = _parse_new_line();  
                // <= new_line  
                _ch = ch1;  
                _cursor = pos1;   
                _testing = testing1;  
                $$ = null;  
                success = !success;  
                // <= !new_line  
                if (!success) break;  
                var seq = new List(5)..[0] = $$;  
                // => !identifier_base  
                var ch2 = _ch, pos2 = _cursor, testing2 = _testing;   
                _testing = _inputLen + 1;  
                // => identifier_base  
                $$ = _parse_identifier_base();  
                // <= identifier_base  
                _ch = ch2;  
                _cursor = pos2;   
                _testing = testing2;  
                $$ = null;  
                success = !success;  
                // <= !identifier_base  
                if (!success) break;  
                seq[1] = $$;  
                // => !string_literal_base  
                var ch3 = _ch, pos3 = _cursor, testing3 = _testing;   
                _testing = _inputLen + 1;  
                // => string_literal_base  
                $$ = _parse_string_literal_base();  
                // <= string_literal_base  
                _ch = ch3;  
                _cursor = pos3;   
                _testing = testing3;  
                $$ = null;  
                success = !success;  
                // <= !string_literal_base  
                if (!success) break;  
                seq[2] = $$;  
                // => !constant_base  
                var ch4 = _ch, pos4 = _cursor, testing4 = _testing;   
                _testing = _inputLen + 1;  
                // => constant_base  
                $$ = _parse_constant_base();  
                // <= constant_base  
                _ch = ch4;  
                _cursor = pos4;   
                _testing = testing4;  
                $$ = null;  
                success = !success;  
                // <= !constant_base  
                if (!success) break;  
                seq[3] = $$;  
                // => .  
                $$ = _matchAny();  
                // <= .  
                if (!success) break;  
                seq[4] = $$;  
                $$ = seq;  
                break;  
              }  
              if (!success) {  
                _ch = ch0;  
                _cursor = pos0;  
              }  
              _startPos = startPos1;  
              // <= !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
              break;  
            // No matches  
            case 1:  
              $$ = null;  
              success = false;  
              break;  
          }  
          if (!success && _cursor > _testing) {  
            // Expected: null  
            _failure(_expect104);  
          }  
          // <= (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
          if (success) {
           if (first) {      
              first = false;
              reps = [$$];
              testing0 = _testing;                  
            } else {
              reps.add($$);
            }
            _testing = _cursor;   
          } else {
            success = !first;
            if (success) {      
              _testing = testing0;
              $$ = reps;      
            } else $$ = null;
            break;
          }  
        }
        // <= (!new_line !identifier_base !string_literal_base !constant_base .)+
        if (success) {    
          // (!new_line !identifier_base !string_literal_base !constant_base .)+
          final $1 = $$;
          final $start = startPos0;
          $$ = new SourceFragment(position: $start, text: _text());
        }
        _startPos = startPos0;
        break;
      // [\"]
      case 1:
        while (true) {
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => string_literal_base
          $$ = _parse_string_literal_base();
          // <= string_literal_base
          if (success) {    
            // string_literal_base
            final $1 = $$;
            final $start = startPos2;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos2;
          if (success) break;
          var startPos3 = _startPos;
          _startPos = _cursor;
          // => (!new_line !identifier_base !string_literal_base !constant_base .)+
          var testing5;
          for (var first = true, reps; ;) {  
            // => (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {  
              // [\u0000-\u0010ffff]  
              // EOF  
              case 0:  
              case 2:  
                // => !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                var ch5 = _ch, pos5 = _cursor, startPos4 = _startPos;  
                _startPos = _cursor;  
                while (true) {    
                  // => !new_line  
                  var ch6 = _ch, pos6 = _cursor, testing6 = _testing;   
                  _testing = _inputLen + 1;  
                  // => new_line  
                  $$ = _parse_new_line();  
                  // <= new_line  
                  _ch = ch6;  
                  _cursor = pos6;   
                  _testing = testing6;  
                  $$ = null;  
                  success = !success;  
                  // <= !new_line  
                  if (!success) break;  
                  var seq = new List(5)..[0] = $$;  
                  // => !identifier_base  
                  var ch7 = _ch, pos7 = _cursor, testing7 = _testing;   
                  _testing = _inputLen + 1;  
                  // => identifier_base  
                  $$ = _parse_identifier_base();  
                  // <= identifier_base  
                  _ch = ch7;  
                  _cursor = pos7;   
                  _testing = testing7;  
                  $$ = null;  
                  success = !success;  
                  // <= !identifier_base  
                  if (!success) break;  
                  seq[1] = $$;  
                  // => !string_literal_base  
                  var ch8 = _ch, pos8 = _cursor, testing8 = _testing;   
                  _testing = _inputLen + 1;  
                  // => string_literal_base  
                  $$ = _parse_string_literal_base();  
                  // <= string_literal_base  
                  _ch = ch8;  
                  _cursor = pos8;   
                  _testing = testing8;  
                  $$ = null;  
                  success = !success;  
                  // <= !string_literal_base  
                  if (!success) break;  
                  seq[2] = $$;  
                  // => !constant_base  
                  var ch9 = _ch, pos9 = _cursor, testing9 = _testing;   
                  _testing = _inputLen + 1;  
                  // => constant_base  
                  $$ = _parse_constant_base();  
                  // <= constant_base  
                  _ch = ch9;  
                  _cursor = pos9;   
                  _testing = testing9;  
                  $$ = null;  
                  success = !success;  
                  // <= !constant_base  
                  if (!success) break;  
                  seq[3] = $$;  
                  // => .  
                  $$ = _matchAny();  
                  // <= .  
                  if (!success) break;  
                  seq[4] = $$;  
                  $$ = seq;  
                  break;  
                }  
                if (!success) {  
                  _ch = ch5;  
                  _cursor = pos5;  
                }  
                _startPos = startPos4;  
                // <= !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                break;  
              // No matches  
              case 1:  
                $$ = null;  
                success = false;  
                break;  
            }  
            if (!success && _cursor > _testing) {  
              // Expected: null  
              _failure(_expect104);  
            }  
            // <= (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            if (success) {
             if (first) {      
                first = false;
                reps = [$$];
                testing5 = _testing;                  
              } else {
                reps.add($$);
              }
              _testing = _cursor;   
            } else {
              success = !first;
              if (success) {      
                _testing = testing5;
                $$ = reps;      
              } else $$ = null;
              break;
            }  
          }
          // <= (!new_line !identifier_base !string_literal_base !constant_base .)+
          if (success) {    
            // (!new_line !identifier_base !string_literal_base !constant_base .)+
            final $1 = $$;
            final $start = startPos3;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos3;
          break;
        }
        break;
      // [\'] [.] [0-9]
      case 2:
        while (true) {
          var startPos5 = _startPos;
          _startPos = _cursor;
          // => constant_base
          $$ = _parse_constant_base();
          // <= constant_base
          if (success) {    
            // constant_base
            final $1 = $$;
            final $start = startPos5;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos5;
          if (success) break;
          var startPos6 = _startPos;
          _startPos = _cursor;
          // => (!new_line !identifier_base !string_literal_base !constant_base .)+
          var testing10;
          for (var first = true, reps; ;) {  
            // => (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {  
              // [\u0000-\u0010ffff]  
              // EOF  
              case 0:  
              case 2:  
                // => !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                var ch10 = _ch, pos10 = _cursor, startPos7 = _startPos;  
                _startPos = _cursor;  
                while (true) {    
                  // => !new_line  
                  var ch11 = _ch, pos11 = _cursor, testing11 = _testing;   
                  _testing = _inputLen + 1;  
                  // => new_line  
                  $$ = _parse_new_line();  
                  // <= new_line  
                  _ch = ch11;  
                  _cursor = pos11;   
                  _testing = testing11;  
                  $$ = null;  
                  success = !success;  
                  // <= !new_line  
                  if (!success) break;  
                  var seq = new List(5)..[0] = $$;  
                  // => !identifier_base  
                  var ch12 = _ch, pos12 = _cursor, testing12 = _testing;   
                  _testing = _inputLen + 1;  
                  // => identifier_base  
                  $$ = _parse_identifier_base();  
                  // <= identifier_base  
                  _ch = ch12;  
                  _cursor = pos12;   
                  _testing = testing12;  
                  $$ = null;  
                  success = !success;  
                  // <= !identifier_base  
                  if (!success) break;  
                  seq[1] = $$;  
                  // => !string_literal_base  
                  var ch13 = _ch, pos13 = _cursor, testing13 = _testing;   
                  _testing = _inputLen + 1;  
                  // => string_literal_base  
                  $$ = _parse_string_literal_base();  
                  // <= string_literal_base  
                  _ch = ch13;  
                  _cursor = pos13;   
                  _testing = testing13;  
                  $$ = null;  
                  success = !success;  
                  // <= !string_literal_base  
                  if (!success) break;  
                  seq[2] = $$;  
                  // => !constant_base  
                  var ch14 = _ch, pos14 = _cursor, testing14 = _testing;   
                  _testing = _inputLen + 1;  
                  // => constant_base  
                  $$ = _parse_constant_base();  
                  // <= constant_base  
                  _ch = ch14;  
                  _cursor = pos14;   
                  _testing = testing14;  
                  $$ = null;  
                  success = !success;  
                  // <= !constant_base  
                  if (!success) break;  
                  seq[3] = $$;  
                  // => .  
                  $$ = _matchAny();  
                  // <= .  
                  if (!success) break;  
                  seq[4] = $$;  
                  $$ = seq;  
                  break;  
                }  
                if (!success) {  
                  _ch = ch10;  
                  _cursor = pos10;  
                }  
                _startPos = startPos7;  
                // <= !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                break;  
              // No matches  
              case 1:  
                $$ = null;  
                success = false;  
                break;  
            }  
            if (!success && _cursor > _testing) {  
              // Expected: null  
              _failure(_expect104);  
            }  
            // <= (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            if (success) {
             if (first) {      
                first = false;
                reps = [$$];
                testing10 = _testing;                  
              } else {
                reps.add($$);
              }
              _testing = _cursor;   
            } else {
              success = !first;
              if (success) {      
                _testing = testing10;
                $$ = reps;      
              } else $$ = null;
              break;
            }  
          }
          // <= (!new_line !identifier_base !string_literal_base !constant_base .)+
          if (success) {    
            // (!new_line !identifier_base !string_literal_base !constant_base .)+
            final $1 = $$;
            final $start = startPos6;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos6;
          break;
        }
        break;
      // [A-K] [M-Z] [\\] [_] [a-z]
      case 3:
        while (true) {
          var startPos8 = _startPos;
          _startPos = _cursor;
          // => identifier_base
          $$ = _parse_identifier_base();
          // <= identifier_base
          if (success) {    
            // identifier_base
            final $1 = $$;
            final $start = startPos8;
            $$ = new SourceFragment(name: _text(), position: $start, text: _text());
          }
          _startPos = startPos8;
          if (success) break;
          var startPos9 = _startPos;
          _startPos = _cursor;
          // => (!new_line !identifier_base !string_literal_base !constant_base .)+
          var testing15;
          for (var first = true, reps; ;) {  
            // => (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {  
              // [\u0000-\u0010ffff]  
              // EOF  
              case 0:  
              case 2:  
                // => !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                var ch15 = _ch, pos15 = _cursor, startPos10 = _startPos;  
                _startPos = _cursor;  
                while (true) {    
                  // => !new_line  
                  var ch16 = _ch, pos16 = _cursor, testing16 = _testing;   
                  _testing = _inputLen + 1;  
                  // => new_line  
                  $$ = _parse_new_line();  
                  // <= new_line  
                  _ch = ch16;  
                  _cursor = pos16;   
                  _testing = testing16;  
                  $$ = null;  
                  success = !success;  
                  // <= !new_line  
                  if (!success) break;  
                  var seq = new List(5)..[0] = $$;  
                  // => !identifier_base  
                  var ch17 = _ch, pos17 = _cursor, testing17 = _testing;   
                  _testing = _inputLen + 1;  
                  // => identifier_base  
                  $$ = _parse_identifier_base();  
                  // <= identifier_base  
                  _ch = ch17;  
                  _cursor = pos17;   
                  _testing = testing17;  
                  $$ = null;  
                  success = !success;  
                  // <= !identifier_base  
                  if (!success) break;  
                  seq[1] = $$;  
                  // => !string_literal_base  
                  var ch18 = _ch, pos18 = _cursor, testing18 = _testing;   
                  _testing = _inputLen + 1;  
                  // => string_literal_base  
                  $$ = _parse_string_literal_base();  
                  // <= string_literal_base  
                  _ch = ch18;  
                  _cursor = pos18;   
                  _testing = testing18;  
                  $$ = null;  
                  success = !success;  
                  // <= !string_literal_base  
                  if (!success) break;  
                  seq[2] = $$;  
                  // => !constant_base  
                  var ch19 = _ch, pos19 = _cursor, testing19 = _testing;   
                  _testing = _inputLen + 1;  
                  // => constant_base  
                  $$ = _parse_constant_base();  
                  // <= constant_base  
                  _ch = ch19;  
                  _cursor = pos19;   
                  _testing = testing19;  
                  $$ = null;  
                  success = !success;  
                  // <= !constant_base  
                  if (!success) break;  
                  seq[3] = $$;  
                  // => .  
                  $$ = _matchAny();  
                  // <= .  
                  if (!success) break;  
                  seq[4] = $$;  
                  $$ = seq;  
                  break;  
                }  
                if (!success) {  
                  _ch = ch15;  
                  _cursor = pos15;  
                }  
                _startPos = startPos10;  
                // <= !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                break;  
              // No matches  
              case 1:  
                $$ = null;  
                success = false;  
                break;  
            }  
            if (!success && _cursor > _testing) {  
              // Expected: null  
              _failure(_expect104);  
            }  
            // <= (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            if (success) {
             if (first) {      
                first = false;
                reps = [$$];
                testing15 = _testing;                  
              } else {
                reps.add($$);
              }
              _testing = _cursor;   
            } else {
              success = !first;
              if (success) {      
                _testing = testing15;
                $$ = reps;      
              } else $$ = null;
              break;
            }  
          }
          // <= (!new_line !identifier_base !string_literal_base !constant_base .)+
          if (success) {    
            // (!new_line !identifier_base !string_literal_base !constant_base .)+
            final $1 = $$;
            final $start = startPos9;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos9;
          break;
        }
        break;
      // [L]
      case 4:
        while (true) {
          var startPos11 = _startPos;
          _startPos = _cursor;
          // => string_literal_base
          $$ = _parse_string_literal_base();
          // <= string_literal_base
          if (success) {    
            // string_literal_base
            final $1 = $$;
            final $start = startPos11;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos11;
          if (success) break;
          var startPos12 = _startPos;
          _startPos = _cursor;
          // => constant_base
          $$ = _parse_constant_base();
          // <= constant_base
          if (success) {    
            // constant_base
            final $1 = $$;
            final $start = startPos12;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos12;
          if (success) break;
          var startPos13 = _startPos;
          _startPos = _cursor;
          // => identifier_base
          $$ = _parse_identifier_base();
          // <= identifier_base
          if (success) {    
            // identifier_base
            final $1 = $$;
            final $start = startPos13;
            $$ = new SourceFragment(name: _text(), position: $start, text: _text());
          }
          _startPos = startPos13;
          if (success) break;
          var startPos14 = _startPos;
          _startPos = _cursor;
          // => (!new_line !identifier_base !string_literal_base !constant_base .)+
          var testing20;
          for (var first = true, reps; ;) {  
            // => (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {  
              // [\u0000-\u0010ffff]  
              // EOF  
              case 0:  
              case 2:  
                // => !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                var ch20 = _ch, pos20 = _cursor, startPos15 = _startPos;  
                _startPos = _cursor;  
                while (true) {    
                  // => !new_line  
                  var ch21 = _ch, pos21 = _cursor, testing21 = _testing;   
                  _testing = _inputLen + 1;  
                  // => new_line  
                  $$ = _parse_new_line();  
                  // <= new_line  
                  _ch = ch21;  
                  _cursor = pos21;   
                  _testing = testing21;  
                  $$ = null;  
                  success = !success;  
                  // <= !new_line  
                  if (!success) break;  
                  var seq = new List(5)..[0] = $$;  
                  // => !identifier_base  
                  var ch22 = _ch, pos22 = _cursor, testing22 = _testing;   
                  _testing = _inputLen + 1;  
                  // => identifier_base  
                  $$ = _parse_identifier_base();  
                  // <= identifier_base  
                  _ch = ch22;  
                  _cursor = pos22;   
                  _testing = testing22;  
                  $$ = null;  
                  success = !success;  
                  // <= !identifier_base  
                  if (!success) break;  
                  seq[1] = $$;  
                  // => !string_literal_base  
                  var ch23 = _ch, pos23 = _cursor, testing23 = _testing;   
                  _testing = _inputLen + 1;  
                  // => string_literal_base  
                  $$ = _parse_string_literal_base();  
                  // <= string_literal_base  
                  _ch = ch23;  
                  _cursor = pos23;   
                  _testing = testing23;  
                  $$ = null;  
                  success = !success;  
                  // <= !string_literal_base  
                  if (!success) break;  
                  seq[2] = $$;  
                  // => !constant_base  
                  var ch24 = _ch, pos24 = _cursor, testing24 = _testing;   
                  _testing = _inputLen + 1;  
                  // => constant_base  
                  $$ = _parse_constant_base();  
                  // <= constant_base  
                  _ch = ch24;  
                  _cursor = pos24;   
                  _testing = testing24;  
                  $$ = null;  
                  success = !success;  
                  // <= !constant_base  
                  if (!success) break;  
                  seq[3] = $$;  
                  // => .  
                  $$ = _matchAny();  
                  // <= .  
                  if (!success) break;  
                  seq[4] = $$;  
                  $$ = seq;  
                  break;  
                }  
                if (!success) {  
                  _ch = ch20;  
                  _cursor = pos20;  
                }  
                _startPos = startPos15;  
                // <= !new_line !identifier_base !string_literal_base !constant_base . # Sequence  
                break;  
              // No matches  
              case 1:  
                $$ = null;  
                success = false;  
                break;  
            }  
            if (!success && _cursor > _testing) {  
              // Expected: null  
              _failure(_expect104);  
            }  
            // <= (!new_line !identifier_base !string_literal_base !constant_base .) # Choice  
            if (success) {
             if (first) {      
                first = false;
                reps = [$$];
                testing20 = _testing;                  
              } else {
                reps.add($$);
              }
              _testing = _cursor;   
            } else {
              success = !first;
              if (success) {      
                _testing = testing20;
                $$ = reps;      
              } else $$ = null;
              break;
            }  
          }
          // <= (!new_line !identifier_base !string_literal_base !constant_base .)+
          if (success) {    
            // (!new_line !identifier_base !string_literal_base !constant_base .)+
            final $1 = $$;
            final $start = startPos14;
            $$ = new SourceFragment(position: $start, text: _text());
          }
          _startPos = startPos14;
          break;
        }
        break;
      // No matches
      case 5:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: source_fragment
      _failure(_expect103);
    }
    // <= string_literal_base / constant_base / identifier_base / (!new_line !identifier_base !string_literal_base !constant_base .)+ # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_source_line() {
    // MORPHEME
    // source_line <- source_fragment+ new_line_fragment? / new_line_fragment
    var $$;
    _beginToken(68);  
    // => source_fragment+ new_line_fragment? / new_line_fragment # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      case 0:
        while (true) {
          // => source_fragment+ new_line_fragment? # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => source_fragment+
            var testing0;
            for (var first = true, reps; ;) {  
              // => source_fragment  
              $$ = _parse_source_fragment();  
              // <= source_fragment  
              if (success) {
               if (first) {      
                  first = false;
                  reps = [$$];
                  testing0 = _testing;                  
                } else {
                  reps.add($$);
                }
                _testing = _cursor;   
              } else {
                success = !first;
                if (success) {      
                  _testing = testing0;
                  $$ = reps;      
                } else $$ = null;
                break;
              }  
            }
            // <= source_fragment+
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => new_line_fragment?
            var testing1 = _testing;
            _testing = _cursor;
            // => new_line_fragment
            $$ = _parse_new_line_fragment();
            // <= new_line_fragment
            success = true; 
            _testing = testing1;
            // <= new_line_fragment?
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // source_fragment+
              final $1 = seq[0];
              // new_line_fragment?
              final $2 = seq[1];
              final $start = startPos0;
              $$ = _sourceLine($1, $2, $start);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= source_fragment+ new_line_fragment? # Sequence
          if (success) break;
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => new_line_fragment
          $$ = _parse_new_line_fragment();
          // <= new_line_fragment
          if (success) {    
            // new_line_fragment
            final $1 = $$;
            final $start = startPos1;
            $$ = _sourceLine([$1], null, $start);
          }
          _startPos = startPos1;
          break;
        }
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
      // EOF
      case 2:
        // => source_fragment+ new_line_fragment? # Sequence
        var ch1 = _ch, pos1 = _cursor, startPos2 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => source_fragment+
          var testing2;
          for (var first = true, reps; ;) {  
            // => source_fragment  
            $$ = _parse_source_fragment();  
            // <= source_fragment  
            if (success) {
             if (first) {      
                first = false;
                reps = [$$];
                testing2 = _testing;                  
              } else {
                reps.add($$);
              }
              _testing = _cursor;   
            } else {
              success = !first;
              if (success) {      
                _testing = testing2;
                $$ = reps;      
              } else $$ = null;
              break;
            }  
          }
          // <= source_fragment+
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => new_line_fragment?
          var testing3 = _testing;
          _testing = _cursor;
          // => new_line_fragment
          $$ = _parse_new_line_fragment();
          // <= new_line_fragment
          success = true; 
          _testing = testing3;
          // <= new_line_fragment?
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // source_fragment+
            final $1 = seq[0];
            // new_line_fragment?
            final $2 = seq[1];
            final $start = startPos2;
            $$ = _sourceLine($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch1;
          _cursor = pos1;
        }
        _startPos = startPos2;
        // <= source_fragment+ new_line_fragment? # Sequence
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: source_line
      _failure(_expect105);
    }
    // <= source_fragment+ new_line_fragment? / new_line_fragment # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_spaces() {
    // MORPHEME
    // spaces <- [\t ]*
    var $$;
    _beginToken(69);  
    // => [\t ]* # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [\t ]*
        var testing0 = _testing; 
        for (var reps = []; ; ) {
          _testing = _cursor;
          // => [\t ]
          $$ = _matchMapping(9, 32, _mapping9);
          // <= [\t ]
          if (success) {  
            reps.add($$);
          } else {
            success = true;
            _testing = testing0;
            $$ = reps;
            break; 
          }
        }
        // <= [\t ]*
        if (success) {    
          // [\t ]*
          final $1 = $$;
          final $start = startPos0;
          $$ = _text();
        }
        _startPos = startPos0;
        break;
      // No matches
      case 1:
        $$ = null;
        success = true;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: spaces
      _failure(_expect106);
    }
    // <= [\t ]* # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_string_literal_base() {
    // MORPHEME
    // string_literal_base <- '"' s_char_sequence? '"' / 'L"' s_char_sequence? '"'
    var $$;
    _beginToken(70);  
    // => '"' s_char_sequence? '"' / 'L"' s_char_sequence? '"' # Choice
    switch (_getState(_transitions44)) {
      // [\"]
      case 0:
        // => '"' s_char_sequence? '"' # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '"'
          $$ = '\"';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '"'
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => s_char_sequence?
          var testing0 = _testing;
          _testing = _cursor;
          // => s_char_sequence
          $$ = _parse_s_char_sequence();
          // <= s_char_sequence
          success = true; 
          _testing = testing0;
          // <= s_char_sequence?
          if (!success) break;
          seq[1] = $$;
          // => '"'
          $$ = _matchChar(34, '\"');
          // <= '"'
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // '"'
            final $1 = seq[0];
            // s_char_sequence?
            final $2 = seq[1];
            // '"'
            final $3 = seq[2];
            final $start = startPos0;
            $$ = _stringLiteral(_text(), $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '"' s_char_sequence? '"' # Sequence
        break;
      // [L]
      case 1:
        // => 'L"' s_char_sequence? '"' # Sequence
        var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => 'L"'
          $$ = _matchString(_strings23, 'L\"');
          // <= 'L"'
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => s_char_sequence?
          var testing1 = _testing;
          _testing = _cursor;
          // => s_char_sequence
          $$ = _parse_s_char_sequence();
          // <= s_char_sequence
          success = true; 
          _testing = testing1;
          // <= s_char_sequence?
          if (!success) break;
          seq[1] = $$;
          // => '"'
          $$ = _matchChar(34, '\"');
          // <= '"'
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // 'L"'
            final $1 = seq[0];
            // s_char_sequence?
            final $2 = seq[1];
            // '"'
            final $3 = seq[2];
            final $start = startPos1;
            $$ = _stringLiteral(_text(), $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch1;
          _cursor = pos1;
        }
        _startPos = startPos1;
        // <= 'L"' s_char_sequence? '"' # Sequence
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: string_literal_base
      _failure(_expect107);
    }
    // <= '"' s_char_sequence? '"' / 'L"' s_char_sequence? '"' # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_text_line() {
    // LEXEME
    // text_line <- !'#' source_line
    var $$;
    // => !'#' source_line # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        // => !'#' source_line # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => !'#'
          var ch1 = _ch, pos1 = _cursor, testing0 = _testing; 
          _testing = _inputLen + 1;
          // => '#'
          $$ = _matchChar(35, '#');
          // <= '#'
          _ch = ch1;
          _cursor = pos1; 
          _testing = testing0;
          $$ = null;
          success = !success;
          // <= !'#'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => source_line
          $$ = _parse_source_line();
          // <= source_line
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // !'#'
            final $1 = seq[0];
            // source_line
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $2;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= !'#' source_line # Sequence
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: text_line
      _failure(_expect39);
    }
    // <= !'#' source_line # Choice
    return $$;
  }
  
  dynamic _parse_tilde() {
    // MORPHEME
    // tilde <- '~' spaces
    var $$;
    _beginToken(71);  
    // => '~' spaces # Choice
    switch (_ch == 126 ? 0 : _ch == -1 ? 2 : 1) {
      // [~]
      case 0:
        // => '~' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '~'
          $$ = '~';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '~'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '~'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '~' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '~'
      _failure(_expect108);
    }
    // <= '~' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_unary_expression() {
    // NONTERMINAL
    // unary_expression <- primary_expression / unary_operator unary_expression
    var $$;          
    var pos = _cursor;    
    if(_cachePos[17] >= pos) {
      $$ = _getFromCache(17);
    } else {
      _cachePos[17] = pos;
    }
    if($$ != null) {
      return $$[0];       
    }  
    // => primary_expression / unary_operator unary_expression # Choice
    switch (_getState(_transitions2)) {
      // [!] [+] [~]
      case 0:
        // => unary_operator unary_expression # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => unary_operator
          $$ = _parse_unary_operator();
          // <= unary_operator
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => unary_expression
          $$ = _parse_unary_expression();
          // <= unary_expression
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // unary_operator
            final $1 = seq[0];
            // unary_expression
            final $2 = seq[1];
            final $start = startPos0;
            $$ = _unary($1, $2, $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= unary_operator unary_expression # Sequence
        break;
      // [\'-(] [.] [0-9] [A-Z] [\\] [_] [a-z]
      case 1:
        var startPos1 = _startPos;
        _startPos = _cursor;
        // => primary_expression
        $$ = _parse_primary_expression();
        // <= primary_expression
        _startPos = startPos1;
        break;
      // No matches
      // EOF
      case 2:
      case 3:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= primary_expression / unary_operator unary_expression # Choice
    if (_cacheable[17]) {
      _addToCache($$, pos, 17);
    }  
    return $$;
  }
  
  dynamic _parse_unary_operator() {
    // LEXEME
    // unary_operator <- plus / minus / tilde / exclamation
    var $$;
    // => plus / minus / tilde / exclamation # Choice
    switch (_getState(_transitions18)) {
      // [!]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => exclamation
        $$ = _parse_exclamation();
        // <= exclamation
        _startPos = startPos0;
        break;
      // [+]
      case 1:
        while (true) {
          var startPos1 = _startPos;
          _startPos = _cursor;
          // => plus
          $$ = _parse_plus();
          // <= plus
          _startPos = startPos1;
          if (success) break;
          var startPos2 = _startPos;
          _startPos = _cursor;
          // => minus
          $$ = _parse_minus();
          // <= minus
          _startPos = startPos2;
          break;
        }
        break;
      // [~]
      case 2:
        var startPos3 = _startPos;
        _startPos = _cursor;
        // => tilde
        $$ = _parse_tilde();
        // <= tilde
        _startPos = startPos3;
        break;
      // No matches
      // EOF
      case 3:
      case 4:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: unary_operator
      _failure(_expect40);
    }
    // <= plus / minus / tilde / exclamation # Choice
    return $$;
  }
  
  dynamic _parse_undef() {
    // MORPHEME
    // undef <- '#undef' spaces
    var $$;
    _beginToken(72);  
    // => '#undef' spaces # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => '#undef' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '#undef'
          $$ = _matchString(_strings24, '#undef');
          // <= '#undef'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '#undef'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '#undef' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#undef'
      _failure(_expect109);
    }
    // <= '#undef' spaces # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_undef_directive() {
    // MORPHEME
    // undef_directive <- undef identifier new_line?
    var $$;
    _beginToken(73);  
    // => undef identifier new_line? # Choice
    switch (_ch == 35 ? 0 : _ch == -1 ? 2 : 1) {
      // [#]
      case 0:
        // => undef identifier new_line? # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => undef
          $$ = _parse_undef();
          // <= undef
          if (!success) break;
          var seq = new List(3)..[0] = $$;
          // => identifier
          $$ = _parse_identifier();
          // <= identifier
          if (!success) break;
          seq[1] = $$;
          // => new_line?
          var testing0 = _testing;
          _testing = _cursor;
          // => new_line
          $$ = _parse_new_line();
          // <= new_line
          success = true; 
          _testing = testing0;
          // <= new_line?
          if (!success) break;
          seq[2] = $$;
          $$ = seq;
          if (success) {    
            // undef
            final $1 = seq[0];
            // identifier
            final $2 = seq[1];
            // new_line?
            final $3 = seq[2];
            final $start = startPos0;
            $$ = new UndefDirective(identifier: $2, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= undef identifier new_line? # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '#undef'
      _failure(_expect109);
    }
    // <= undef identifier new_line? # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_universal_character_name() {
    // MORPHEME
    // universal_character_name <- '\\U' hex_quad hex_quad / '\\u' hex_quad
    var $$;
    _beginToken(74);  
    // => '\\U' hex_quad hex_quad / '\\u' hex_quad # Choice
    switch (_ch == 92 ? 0 : _ch == -1 ? 2 : 1) {
      // [\\]
      case 0:
        while (true) {
          // => '\\U' hex_quad hex_quad # Sequence
          var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => '\\U'
            $$ = _matchString(_strings25, '\\U');
            // <= '\\U'
            if (!success) break;
            var seq = new List(3)..[0] = $$;
            // => hex_quad
            $$ = _parse_hex_quad();
            // <= hex_quad
            if (!success) break;
            seq[1] = $$;
            // => hex_quad
            $$ = _parse_hex_quad();
            // <= hex_quad
            if (!success) break;
            seq[2] = $$;
            $$ = seq;
            if (success) {    
              // '\\U'
              final $1 = seq[0];
              // hex_quad
              final $2 = seq[1];
              // hex_quad
              final $3 = seq[2];
              final $start = startPos0;
              $$ = int.parse(_text(2), radix: 16);
            }
            break;
          }
          if (!success) {
            _ch = ch0;
            _cursor = pos0;
          }
          _startPos = startPos0;
          // <= '\\U' hex_quad hex_quad # Sequence
          if (success) break;
          // => '\\u' hex_quad # Sequence
          var ch1 = _ch, pos1 = _cursor, startPos1 = _startPos;
          _startPos = _cursor;
          while (true) {  
            // => '\\u'
            $$ = _matchString(_strings26, '\\u');
            // <= '\\u'
            if (!success) break;
            var seq = new List(2)..[0] = $$;
            // => hex_quad
            $$ = _parse_hex_quad();
            // <= hex_quad
            if (!success) break;
            seq[1] = $$;
            $$ = seq;
            if (success) {    
              // '\\u'
              final $1 = seq[0];
              // hex_quad
              final $2 = seq[1];
              final $start = startPos1;
              $$ = int.parse(_text(2), radix: 16);
            }
            break;
          }
          if (!success) {
            _ch = ch1;
            _cursor = pos1;
          }
          _startPos = startPos1;
          // <= '\\u' hex_quad # Sequence
          break;
        }
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: universal_character_name
      _failure(_expect110);
    }
    // <= '\\U' hex_quad hex_quad / '\\u' hex_quad # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_unsigned_suffix() {
    // MORPHEME
    // unsigned_suffix <- [Uu]
    var $$;
    _beginToken(75);  
    // => [Uu] # Choice
    switch (_getState(_transitions45)) {
      // [U] [u]
      case 0:
        var startPos0 = _startPos;
        _startPos = _cursor;
        // => [Uu]
        $$ = _matchMapping(85, 117, _mapping10);
        // <= [Uu]
        _startPos = startPos0;
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: unsigned_suffix
      _failure(_expect111);
    }
    // <= [Uu] # Choice
    _endToken();
    return $$;
  }
  
  dynamic _parse_vertical_line() {
    // LEXEME
    // vertical_line <- '|' spaces
    var $$;
    // => '|' spaces # Choice
    switch (_ch == 124 ? 0 : _ch == -1 ? 2 : 1) {
      // [|]
      case 0:
        // => '|' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '|'
          $$ = '|';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '|'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '|'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '|' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '|'
      _failure(_expect41);
    }
    // <= '|' spaces # Choice
    return $$;
  }
  
  dynamic _parse_vertical_line2() {
    // LEXEME
    // vertical_line2 <- '||' spaces
    var $$;
    // => '||' spaces # Choice
    switch (_ch == 124 ? 0 : _ch == -1 ? 2 : 1) {
      // [|]
      case 0:
        // => '||' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '||'
          $$ = _matchString(_strings10, '||');
          // <= '||'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '||'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '||' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '||'
      _failure(_expect42);
    }
    // <= '||' spaces # Choice
    return $$;
  }
  
  dynamic _parse_xor() {
    // LEXEME
    // xor <- '^' spaces
    var $$;
    // => '^' spaces # Choice
    switch (_ch == 94 ? 0 : _ch == -1 ? 2 : 1) {
      // [^]
      case 0:
        // => '^' spaces # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => '^'
          $$ = '^';
          success = true;
          if (++_cursor < _inputLen) {
            _ch = _input[_cursor];
          } else {
            _ch = -1;
          }
          // <= '^'
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => spaces
          $$ = _parse_spaces();
          // <= spaces
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // '^'
            final $1 = seq[0];
            // spaces
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= '^' spaces # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: '^'
      _failure(_expect43);
    }
    // <= '^' spaces # Choice
    return $$;
  }
  
  String _text([int offset = 0]) {
    return new String.fromCharCodes(_input.sublist(_startPos + offset, _cursor));
  }
  
  int _toCodePoint(String string) {
    if (string == null) {
      throw new ArgumentError("string: $string");
    }
  
    var length = string.length;
    if (length == 0) {
      throw new StateError("An empty string contains no elements.");
    }
  
    var start = string.codeUnitAt(0);
    if (length == 1) {
      return start;
    }
  
    if ((start & 0xFC00) == 0xD800) {
      var end = string.codeUnitAt(1);
      if ((end & 0xFC00) == 0xDC00) {
        return (0x10000 + ((start & 0x3FF) << 10) + (end & 0x3FF));
      }
    }
  
    return start;
  }
  
  List<int> _toCodePoints(String string) {
    if (string == null) {
      throw new ArgumentError("string: $string");
    }
  
    var length = string.length;
    if (length == 0) {
      return const <int>[];
    }
  
    var codePoints = <int>[];
    codePoints.length = length;
    var i = 0;
    var pos = 0;
    for ( ; i < length; pos++) {
      var start = string.codeUnitAt(i);
      i++;
      if ((start & 0xFC00) == 0xD800 && i < length) {
        var end = string.codeUnitAt(i);
        if ((end & 0xFC00) == 0xDC00) {
          codePoints[pos] = (0x10000 + ((start & 0x3FF) << 10) + (end & 0x3FF));
          i++;
        } else {
          codePoints[pos] = start;
        }
      } else {
        codePoints[pos] = start;
      }
    }
  
    codePoints.length = pos;
    return codePoints;
  }
  
  static List<bool> _unmap(List<int> mapping) {
    var length = mapping.length;
    var result = new List<bool>(length * 31);
    var offset = 0;
    for (var i = 0; i < length; i++) {
      var v = mapping[i];
      for (var j = 0; j < 31; j++) {
        result[offset++] = v & (1 << j) == 0 ? false : true;
      }
    }
    return result;
  }
  
  List<PParserError> errors() {
    if (success) {
      return <PParserError>[];
    }
  
    String escape(int c) {
      switch (c) {
        case 10:
          return r"\n";
        case 13:
          return r"\r";
        case 09:
          return r"\t";
        case -1:
          return "";
      }
      return new String.fromCharCode(c);
    } 
    
    String getc(int position) {  
      if (position < _inputLen) {
        return "'${escape(_input[position])}'";      
      }       
      return "end of file";
    }
  
    var errors = <PParserError>[];
    if (_failurePos >= _cursor) {
      var set = new Set<PParserError>();
      set.addAll(_errors);
      for (var error in set) {
        if (error.position >= _failurePos) {
          errors.add(error);
        }
      }
      var names = new Set<String>();  
      names.addAll(_expected);
      if (names.contains(null)) {
        var string = getc(_failurePos);
        var message = "Unexpected $string";
        var error = new PParserError(PParserError.UNEXPECTED, _failurePos, _failurePos, message);
        errors.add(error);
      } else {      
        var found = getc(_failurePos);      
        var list = names.toList();
        list.sort();
        var message = "Expected ${list.join(", ")} but found $found";
        var error = new PParserError(PParserError.EXPECTED, _failurePos, _failurePos, message);
        errors.add(error);
      }        
    }
    errors.sort((a, b) => a.position.compareTo(b.position));
    return errors;  
  }
  
  dynamic parse_eval_line() {
    // NONTERMINAL
    // eval_line <- constant_expression (new_line / eof)
    var $$;
    // => constant_expression (new_line / eof) # Choice
    switch (_getState(_transitions1)) {
      // [!] [\'-(] [+] [.] [0-9] [A-Z] [\\] [_] [a-z] [~]
      case 0:
        // => constant_expression (new_line / eof) # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => constant_expression
          $$ = _parse_constant_expression();
          // <= constant_expression
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => (new_line / eof) # Choice
          switch (_getState(_transitions12)) {
            // [\u0000-\t] [\u000b-\u000c] [\u000e-\u0010ffff]
            // EOF
            case 0:
            case 3:
              var startPos1 = _startPos;
              _startPos = _cursor;
              // => eof
              $$ = _parse_eof();
              // <= eof
              _startPos = startPos1;
              break;
            // [\n] [\r]
            case 1:
              while (true) {
                var startPos2 = _startPos;
                _startPos = _cursor;
                // => new_line
                $$ = _parse_new_line();
                // <= new_line
                _startPos = startPos2;
                if (success) break;
                var startPos3 = _startPos;
                _startPos = _cursor;
                // => eof
                $$ = _parse_eof();
                // <= eof
                _startPos = startPos3;
                break;
              }
              break;
            // No matches
            case 2:
              $$ = null;
              success = false;
              break;
          }
          if (!success && _cursor > _testing) {
            // Expected: new_line, eof
            _failure(_expect17);
          }
          // <= (new_line / eof) # Choice
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // constant_expression
            final $1 = seq[0];
            // (new_line / eof)
            final $2 = seq[1];
            final $start = startPos0;
            $$ = $1;
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= constant_expression (new_line / eof) # Sequence
        break;
      // No matches
      // EOF
      case 1:
      case 2:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: 'defined', identifier, constant, '(', unary_operator
      _failure(_expect3);
    }
    // <= constant_expression (new_line / eof) # Choice
    return $$;
  }
  
  dynamic parse_preprocessing_file() {
    // NONTERMINAL
    // preprocessing_file <- group? eof
    var $$;
    // => group? eof # Choice
    switch (_ch >= 0 && _ch <= 1114111 ? 0 : _ch == -1 ? 2 : 1) {
      // [\u0000-\u0010ffff]
      // EOF
      case 0:
      case 2:
        // => group? eof # Sequence
        var ch0 = _ch, pos0 = _cursor, startPos0 = _startPos;
        _startPos = _cursor;
        while (true) {  
          // => group?
          var testing0 = _testing;
          _testing = _cursor;
          // => group
          $$ = _parse_group();
          // <= group
          success = true; 
          _testing = testing0;
          // <= group?
          if (!success) break;
          var seq = new List(2)..[0] = $$;
          // => eof
          $$ = _parse_eof();
          // <= eof
          if (!success) break;
          seq[1] = $$;
          $$ = seq;
          if (success) {    
            // group?
            final $1 = seq[0];
            // eof
            final $2 = seq[1];
            final $start = startPos0;
            $$ = new PreprocessingFile(groups: $1, position: $start);
          }
          break;
        }
        if (!success) {
          _ch = ch0;
          _cursor = pos0;
        }
        _startPos = startPos0;
        // <= group? eof # Sequence
        break;
      // No matches
      case 1:
        $$ = null;
        success = false;
        break;
    }
    if (!success && _cursor > _testing) {
      // Expected: eof
      _failure(_expect0);
    }
    // <= group? eof # Choice
    return $$;
  }
  
  void reset(int pos) {
    if (pos == null) {
      throw new ArgumentError('pos: $pos');
    }
    if (pos < 0 || pos > _inputLen) {
      throw new RangeError('pos');
    }      
    _cursor = pos;
    _cache = new List<Map<int, List>>(136);
    _cachePos = new List<int>.filled(136, -1);  
    _cacheable = new List<bool>.filled(136, false);
    _ch = -1;
    _errors = <PParserError>[];   
    _expected = <String>[];
    _failurePos = -1;
    _startPos = pos;        
    _testing = -1;
    _token = null;
    _tokenLevel = 0;
    _tokenStart = null;  
    if (_cursor < _inputLen) {
      _ch = _input[_cursor];
    }
    success = true;    
  }
  
}

class PParserError {
  static const int EXPECTED = 1;    
      
  static const int MALFORMED = 2;    
      
  static const int MISSING = 3;    
      
  static const int UNEXPECTED = 4;    
      
  static const int UNTERMINATED = 5;    
      
  final int hashCode = 0;
  
  final String message;
  
  final int position;
  
  final int start;
  
  final int type;
  
  PParserError(this.type, this.position, this.start, this.message);
  
  bool operator ==(other) {
    if (identical(this, other)) return true;
    if (other is PParserError) {
      return type == other.type && position == other.position &&
      start == other.start && message == other.message;  
    }
    return false;
  }
  
}

